package cn.kdan.cloud.pdf.office.common.utils;

import org.apache.commons.lang.RandomStringUtils;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.*;

/**
 * 常用日期、自动生成id等方法工具类
 */
public class CommonUtils {

	public final static String DEFAULT_DATE_FORMAT = "yyyy-MM-dd HH:mm:ss";

	public final static String SHORT_DATE_FORMAT = "yyyy-MM-dd";

	public final static String NULL_STR = "";

	public final static String BLANK_STR = " ";

	public final static String UNDERSCORE_STR = "_";

	public final static String SLASH_STR = "/";

	public static final String ENCODING_ISO8859 = "ISO-8859-1";

	public static final String ENCODING_ASCII = "ascii";

	public static final String ENCODING_UTF8 = "utf-8";

	public static final String TEXT_HTML_UTF8 = "text/html; charset=utf-8";

	public static final String APPLICATION_JSON_UTF8 = "application/json; charset=utf-8";

	public static final int SECONDS_A_DAY = 24 * 60 * 60;

	public static final int MILLISECONDS_A_DAY = SECONDS_A_DAY * 1000;

	public static final String ENVIRONMENT = "Environment";

	public static final String IP_REXP = "([1-9]|[1-9]\\d|1\\d{2}|2[0-4]\\d|25[0-5])(\\.(\\d|[1-9]\\d|1\\d{2}|2[0-4]\\d|25[0-5])){3}";

	private static final int DEFAULT_ID_LENGTH = 9;
	/**
	 * 功能描述：获取当前日期时间格式化字符串，默认yyyy-MM-dd HH:mm:ss格式
	 */
	public static String getCurrentDateTime() {
		return getCurrentDateTime(DEFAULT_DATE_FORMAT);
	}

	/**
	 * 功能描述：获取当前日期时间格式化字符串
	 *
	 * @param dateFormat
	 *            日期时间格式
	 */
	public static String getCurrentDateTime(String dateFormat) {
		return new SimpleDateFormat(dateFormat).format(Calendar.getInstance()
				.getTime());
	}

	/**
	 * 功能描述：获取指定日期时间格式化字符串
	 *
	 * @param dateFormat
	 *            日期时间格式
	 */
	public static String getDateTime(String dateFormat, Date date) {
		return new SimpleDateFormat(dateFormat).format(date);
	}


	/**
	 *	 两个日期相差多少分钟
	 * @param endDate
	 * @param startDate
	 * @return
	 */
	public static long getDatePoor(Date endDate, Date startDate) {
		return (endDate.getTime() - startDate.getTime())/(1000 * 60);
	}
	/*
	 * 功能描述：生成对象的主键
	 *
	 * @return String
	 */
	public static String generateId() {
		UUID uuid = UUID.randomUUID();
		String id = uuid.toString();
//		id = id.replaceAll("-", "");
		return id;
	}    // 生成随机 8 位阿拉伯数字用户 ID
	public static String generateRandomUserId() {
		Random random = new Random();
		StringBuilder userId = new StringBuilder();

		for (int i = 0; i < 8; i++) {
			userId.append(random.nextInt(10)); // 生成 0-9 之间的随机数
		}

		return userId.toString();
	}

	/**
	 * 生成随机长度为9的字符串形式的id，使用apache common包中的api
	 * @return 生成的id
	 */
	public static String generateRandomStringId() {
		return generateRandomStringId(DEFAULT_ID_LENGTH);
	}

	/**
	 * 根据传入的参数长度 随机生成字符串
	 * @param count
	 * @return
	 */
	public static String generateRandomStringId(int count) {
		return RandomStringUtils.random(count, true, true);
	}

	/**
	 * 生成Openstack的资源名称
	 * @param prefix
	 * @return
	 */
	public static String generateName(String prefix){
		return prefix + "-" + Long.toHexString((new Date()).getTime());
	}


	/**
	 * 检查参数是否为空 或空串
	 * @param params
	 * @return
	 */
	public static boolean checkParams(String... params){
		if(params == null){
			return false;
		}
		for(String param : params){
			if(StringUtils.isEmpty(param)){
				return false;
			}
		}
		return true;
	}

	/**
	 * 自动生成带有当前日期的字符串
	 * @return
	 */
	public static String generateDateId() {
		Random random = new Random();
		int i = random.nextInt(9999) + 10000;
		String num = String.valueOf(i).substring(1);
		return getCurrentDateTime("yyyyMMddHHmmss") + num;
	}

	/**
	 * String格式的日期转为Date
	 * @param dateFormat 日期格式
	 * @param date
	 * @return
	 * @throws ParseException
	 */
	public static Date parseDate(String dateFormat, String date) throws ParseException {
		return new SimpleDateFormat(dateFormat).parse(date);
	}

	/**
	 * 逗号拼接list
	 * @param list
	 * @return
	 */
	public static String listToString(List<String> list){

		if(list==null){
			return null;
		}

		StringBuilder result = new StringBuilder();
		boolean first = true;

		//第一个前面不拼接","
		for(String string :list) {
			if(first) {
				first=false;
			}else{
				result.append(",");
			}
			result.append(string);
		}
		return result.toString();
	}

	/**
	 * 返回首字母大写后的字符串
	 *
	 * @param str
	 * @return
	 */
	public static String upperFirstLetter(String str){
		if(str.length() > 0){
			return org.apache.commons.lang.StringUtils.upperCase(str.substring(0, 1)) + str.substring(1, str.length());
		}
		return null;
	}



	/**
	 * 用*覆盖内容
	 * @param content
	 * @return
	 */
	private static String coverContent(String content){
		int length = content.length();
		int coverLength = Double.valueOf(Math.floor(length/2)).intValue();
		StringBuilder stringBuilder = new StringBuilder();
		if(coverLength == 0){
			return content;
		}else{
			int halfLength = Double.valueOf(Math.floor((length - coverLength)/2)).intValue();
			for (int i = 0;i < length;i++){
				if(i >= halfLength && (i < (halfLength + coverLength))){
					stringBuilder.append("*");
				}else{
					stringBuilder.append(content.charAt(i));
				}
			}
		}
		return stringBuilder.toString();
	}

	public static String generateVerifyCode() {
		return String.valueOf(new Random().nextInt(899999) + 100000);
	}

}
