package cn.kdan.cloud.pdf.office.email.config;

import cn.kdan.cloud.pdf.office.api.email.constant.MyEmailConstant;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.mail.javamail.JavaMailSenderImpl;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.util.List;
import java.util.Map;
import java.util.Properties;

/**
 * @author ComPDFKit-WPH 2023/1/10
 */
@Slf4j
@Component
@AllArgsConstructor
public class MailSenderConfig {

    private final MailConfig mailConfig;

    private final Map<String, JavaMailSenderImpl> stringListMap;

    private static final boolean enableDebug = true;

    private static String defaultEmail;


    /**
     * 初始化 sender
     */
    @PostConstruct
    public void buildMailSender() {
        List<MailConfig.MailProperties> mailConfigs = mailConfig.getConfigs();
        defaultEmail = mailConfigs.get(0).getUsername();
        log.info("初始化mailSender");
        mailConfigs.forEach(mailProperties -> {
            String protocol = mailProperties.getProtocol();
            String host = mailProperties.getHost();
            // 邮件发送者
            JavaMailSenderImpl javaMailSender = new JavaMailSenderImpl();
            javaMailSender.setDefaultEncoding(mailProperties.getDefaultEncoding());
            javaMailSender.setHost(mailProperties.getHost());
            javaMailSender.setPort(mailProperties.getPort());
            javaMailSender.setProtocol(protocol);
            javaMailSender.setUsername(mailProperties.getUsername());
            javaMailSender.setPassword(mailProperties.getPassword());
            Properties properties = new Properties();

            properties.put("mail.smtp.host", mailProperties.getHost());
            properties.put("mail.smtp.auth", true);
            properties.put("mail.smtp.port", "465");
            properties.put("mail.smtp.socketFactory.port", "465");
            properties.put("mail.smtp.starttls.required", true);
            properties.put("mail.smtp.socketFactory.class", "javax.net.ssl.SSLSocketFactory");
            properties.put("mail.debug", enableDebug);
            javaMailSender.setJavaMailProperties(properties);
            stringListMap.put(mailProperties.getUsername(), javaMailSender);
        });
    }

    /**
     * 获取MailSender
     *
     * @return CustomMailSender
     */
    public JavaMailSenderImpl getSender(String email) {
        if (stringListMap.isEmpty()) {
            buildMailSender();
        }
        // 随机返回一个JavaMailSender
        return stringListMap.get(email);
    }

    /**
     * 添加发送源
     *
     * @param pronMap 配置
     */
    public void addSender(Map<String, Object> pronMap) {
        JavaMailSenderImpl javaMailSender = new JavaMailSenderImpl();
        Properties properties = new Properties();
        properties.putAll(pronMap);
        javaMailSender.setJavaMailProperties(properties);
        stringListMap.put(pronMap.get("username").toString(), javaMailSender);
    }

    /**
     * 获取默认MailSender
     *
     * @return CustomMailSender
     */
    public JavaMailSenderImpl getDefaultSender() {
        if (stringListMap.isEmpty()) {
            buildMailSender();
        }
        // 随机返回一个JavaMailSender
        return stringListMap.get(defaultEmail);
    }

    /**
     * 清理 sender
     */
    public void clear() {
        stringListMap.clear();
    }

}
