package cn.kdan.cloud.pdf.office.payment.interceptor;

import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import lombok.extern.slf4j.Slf4j;
import org.jetbrains.annotations.NotNull;
import org.springframework.http.HttpRequest;
import org.springframework.http.HttpStatus;
import org.springframework.http.client.ClientHttpRequestExecution;
import org.springframework.http.client.ClientHttpRequestInterceptor;
import org.springframework.http.client.ClientHttpResponse;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.nio.charset.StandardCharsets;

/**
 * @author ComPDFKit-WPH 2023/6/28
 * <p>
 * AppStoreClientHttpRequestInterceptor
 */
@Slf4j
@Component
public class GoogleClientHttpRequestInterceptor implements ClientHttpRequestInterceptor {

    @NotNull
    @Override
    public ClientHttpResponse intercept(HttpRequest request, byte[] body, ClientHttpRequestExecution execution) throws IOException {
        //打印请求头信息
        tranceRequest(request, body);
        ClientHttpResponse execute = execution.execute(request, body);
        HttpStatus statusCode = execute.getStatusCode();
        if (statusCode.is2xxSuccessful()) {
            return execute;
        } else if (statusCode.is3xxRedirection()) {
            log.error("Received 3xx Redirection with status code: {}", statusCode.value());
            throw new BackendRuntimeException("Redirection error, status code: " + statusCode.value());
        } else if (statusCode.is4xxClientError()) {
            log.error("Received 4xx Client Error with status code: {}", statusCode.value());
            throw new BackendRuntimeException("Client error, status code: " + statusCode.value());
        } else if (statusCode.is5xxServerError()) {
            log.error("Received 5xx Server Error with status code: {}", statusCode.value());
            throw new BackendRuntimeException("Server error, status code: " + statusCode.value());
        } else {
            log.error("Received unexpected status code: {}", statusCode.value());
            throw new BackendRuntimeException("Unexpected error, status code: " + statusCode.value());
        }
    }

    private void tranceRequest(HttpRequest request, byte[] body) {
        log.info("======= appStore request begin ========");
        log.info("uri : {}", request.getURI());
        log.info("method : {}", request.getMethod());
        log.info("headers : {}", request.getHeaders());
        log.info("request body : {}", new String(body, StandardCharsets.UTF_8));
        log.info("======= appStore request end ========");
    }
}
