package cn.kdan.cloud.pdf.office.payment.interceptor;

import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.payment.properties.PaypalProperties;
import lombok.extern.slf4j.Slf4j;
import org.apache.tomcat.util.codec.binary.Base64;
import org.jetbrains.annotations.NotNull;
import org.springframework.http.HttpRequest;
import org.springframework.http.HttpStatus;
import org.springframework.http.client.ClientHttpRequestExecution;
import org.springframework.http.client.ClientHttpRequestInterceptor;
import org.springframework.http.client.ClientHttpResponse;
import org.springframework.http.client.support.HttpRequestWrapper;
import org.springframework.stereotype.Component;
import org.springframework.web.util.UriComponentsBuilder;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URI;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.util.Date;
import java.util.SortedMap;
import java.util.TreeMap;

/**
 * @author ComPDFKit-WPH 2023/1/16
 */
@Slf4j
@Component
public class PaypalClientHttpRequestInterceptor implements ClientHttpRequestInterceptor {

    private final PaypalProperties properties;

    private final static String BASIC = "Basic ";
    private final static String AUTHORIZATION = "Authorization";

    public PaypalClientHttpRequestInterceptor(PaypalProperties properties) {
        this.properties = properties;
    }

    @NotNull
    @Override
    public ClientHttpResponse intercept(HttpRequest request, byte[] body, ClientHttpRequestExecution execution) throws IOException {
        log.info("===request: {}, {}", request.getURI().getHost(), request.getURI().getPath());

        String auth = properties.getClientId() + ":" + properties.getClientSecret();
        byte[] encodedAuth = Base64.encodeBase64(auth.getBytes(StandardCharsets.US_ASCII));
        String authHeader = BASIC + new String(encodedAuth);
        request.getHeaders().set(AUTHORIZATION, authHeader);
        //打印请求头信息
        tranceRequest(request, body);
        ClientHttpResponse execute = execution.execute(request, body);
        HttpStatus statusCode = execute.getStatusCode();
        if (statusCode.is2xxSuccessful()) {
            return execute;
        } else if (statusCode.is3xxRedirection()) {
            throw new BackendRuntimeException();
        } else if (statusCode.is4xxClientError()) {
            throw new BackendRuntimeException();
        } else if (statusCode.is5xxServerError()) {
            throw new BackendRuntimeException();
        } else {
            throw new BackendRuntimeException();
        }
    }

    private void tranceRequest(HttpRequest request, byte[] body) {
        log.info("======= request begin ========");
        log.info("uri : {}", request.getURI());
        log.info("method : {}", request.getMethod());
        log.info("headers : {}", request.getHeaders());
        log.info("request body : {}", new String(body, StandardCharsets.UTF_8));
        log.info("======= request end ========");
    }

    private static class UriModifyHttpRequestWrapper extends HttpRequestWrapper {
        private final URI uri;

        public UriModifyHttpRequestWrapper(HttpRequest request, URI uri) {
            super(request);
            this.uri = uri;
        }

        @Override
        public URI getURI() {
            return uri;
        }
    }
}
