package cn.kdan.cloud.pdf.office.payment.service;

import cn.kdan.cloud.pdf.office.api.payment.bo.OrderSucceededBO;
import cn.kdan.cloud.pdf.office.api.payment.dto.ChargebackOrderDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreateOrderDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreateOrderManualDTO;
import cn.kdan.cloud.pdf.office.api.payment.vo.CreateOrderResultVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrderUpdateVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrdersVO;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductVO;
import cn.kdan.cloud.pdf.office.common.dto.CreateUserOrderDTO;
import cn.kdan.cloud.pdf.office.payment.entity.Order;
import com.baomidou.mybatisplus.extension.service.IService;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;

/**
 * @author ComPDFKit-WPH 2023-01-29
 */
public interface OrderService extends IService<Order> {

    /**
     * 创建订单
     *
     * @param createOrderDTO createOrderDTO
     * @return CreateOrderResultVO
     */
    CreateOrderResultVO createOrder(CreateOrderDTO createOrderDTO);

    Map<String, String> createUserOrder(CreateUserOrderDTO createUserOrderDTO);

    Map<String, String> createUserSubscription(CreateUserOrderDTO createUserOrderDTO);

    /**
     * 订单创建（applePay专用）
     *
     * @param createOrderDTO createOrderDTO
     * @return CreateOrderResultVO
     */
    @Transactional
    CreateOrderResultVO appStoreCreateOrder(CreateOrderDTO createOrderDTO);

    /**
     * 关闭订单
     *
     * @param orderId orderId
     */
    void closeOrder(String orderId);

    /**
     * 关闭用户同类型订单
     * @param userId
     * @param platform
     */
    void closeOrderByUser(String userId, Integer platform);

    /**
     * 订单支付成功
     *
     * @param orderSucceeded orderSucceeded
     * @return 订单id
     */
    OrderUpdateVO updateOrderSucceeded(OrderSucceededBO orderSucceeded);

    void deleteTheCacheInTheCacheManager(String key);

    /**
     * 通过id获取订单信息
     *
     * @param orderId orderId
     * @return Orders
     */
    OrdersVO getOrderById(String orderId);

    Map<String, String> getStateByOrderId(String orderId);

    List<OrdersVO> getOrderListByStatus(Integer orderStatus, String userId);

    /**
     * 通过自定义订单id获取订单信息
     *
     * @param tradeNo tradeNo
     * @return Orders
     */
    List<OrdersVO> getOrderByTradeNo(String tradeNo);

    ProductVO getProduct(String productId);

    /**
     * 手动创建虚拟订单
     *
     * @param createOrderManual createOrderManual
     * @return OrderUpdateVO
     */
    OrderUpdateVO createOrderManual(CreateOrderManualDTO createOrderManual);

    /**
     * 手动推单
     *
     * @param chargebackOrderDTO chargebackOrderDTO
     */
    void chargebackOrder(ChargebackOrderDTO chargebackOrderDTO);

    /**
     * 新增order
     *
     * @param order order
     */
    void insertOrder(Order order);

    /**
     * 获取订单
     *
     * @param userId userId
     * @param appId appId
     * @return OrdersVO
     */
    List<OrdersVO> getOrderByUserId(String userId,String appId);



    /**
     * 获取订单
     *
     * @param thirdOrderNo thirdOrderNo
     * @return OrdersVO
     */
    OrdersVO getOrderByThirdOrderNo(String thirdOrderNo);

    /**
     * 根据自定义订单号获取订单
     * @param orderNum
     * @return
     */
    List<OrdersVO> getOrderByOrderNum(String orderNum);

    /**
     * 根据第三方id获取订单
     *
     * @param thirdTradeNo 第三方id
     * @return
     */
    List<OrdersVO> getOrderListByThirdTradeNo(String thirdTradeNo);

    Order getOrderByUserIdAndProductId(String userId, String productId);
}
