package cn.kdan.cloud.pdf.office.payment.service;

import cn.kdan.cloud.pdf.office.api.payment.dto.*;
import cn.kdan.cloud.pdf.office.api.payment.enums.PaymentMethodEnum;
import cn.kdan.cloud.pdf.office.api.payment.vo.SubscriptionSchemeResultVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.SubscriptionSchemeVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.SubscriptionsVO;
import cn.kdan.cloud.pdf.office.api.product.enums.PlatformEnum;
import cn.kdan.cloud.pdf.office.payment.entity.Subscription;
import com.baomidou.mybatisplus.extension.service.IService;
import com.baomidou.mybatisplus.core.metadata.IPage;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;

/**
 * @author ComPDFKit-WPH 2023-01-28
 * <p>
 * 生成支付订阅
 */
public interface SubscriptionsService extends IService<Subscription> {

    /**
     * 列表查询
     *
     * @param query 查询参数
     * @return 列表数据
     */
    IPage<Subscription> page(Subscription query);

    boolean judgeTrail(String userId);

    /**
     * 创建订阅信息
     *
     * @param subscriptionInfo subscriptionInfo
     * @return 订阅id
     */
    String createSubscription(CreateSubscriptionDTO subscriptionInfo);

    /**
     * 通过用户id和平台类型 获取最新的订阅信息
     *
     * @param userId   userId
     * @param platform 平台类型
     * @return Subscriptions
     */
    SubscriptionsVO getSubscriptions(String userId, PlatformEnum platform);

    /**
     * 通过用户id和产品id 获取最新的订阅信息
     *
     * @param userId    userId
     * @param productId productId
     * @return Subscriptions
     */
    SubscriptionsVO getSubscriptionsByProductId(String userId, String productId);

    /**
     * 查询最新一条实际支付的订阅信息（排除赠送的订阅）
     * @param userId 用户id
     * @param productId 产品id
     * @return SubscriptionsVO
     */
    SubscriptionsVO getLatestSubscriptionForActualPayment(String userId, String productId);

    /**
     * 获取用户某种产品的下一次自动扣款时间
     * @param userId 用户id
     * @param productId 产品id
     * @return LocalDateTime
     */
    LocalDateTime getNextPayTime(String userId, String productId);

    /**
     * 创建订阅信息 自动续订
     *
     * @param subscriptionManual subscriptionManual
     * @return 订阅id
     */
    String createSubscriptionManual(CreateSubscriptionManualDTO subscriptionManual);


    /**
     * 取消订阅
     *
     * @param chargebackOrderDTO chargebackOrderDTO
     */
    void chargebackSubscription(ChargebackOrderDTO chargebackOrderDTO);

    /**
     * 取消订阅
     *
     * @param cancelSubscriptionDTO cancelSubscriptionDTO
     */
    void chargebackSubscription(CancelSubscriptionDTO cancelSubscriptionDTO);

    /**
     * 查询当前用户的订阅计划和支付订单
     *
     * @param userId       userId
     * @return SubscriptionSchemeVO
     */
    Map<String,SubscriptionSchemeVO> getSubscriptionSchemeInfo(String userId, String appId);

    /**
     * 根据主键id获取
     *
     * @param subscriptionId subscriptionId
     * @return SubscriptionsVO
     */
    SubscriptionsVO getSubscriptionBySubscriptionId(String subscriptionId);

    /**
     * 修改订阅状态
     *
     * @param subscriptionId id
     * @param subscriptionStatus SubscriptionConstant
     */
    void updateSubscriptionStatus(String subscriptionId, Integer subscriptionStatus);

    /**
     * 获取订阅
     *
     * @param userId userId
     * @param productId productId
     * @return List
     */
    List<SubscriptionsVO> getSubscriptionsByUserIdAndProductId(String userId, String productId);

    /**
     * 根据第三方订阅id查询最新的订阅信息
     *
     * @param thirdSubscriptionId thirdSubscriptionId
     * @return SubscriptionsVO
     */
    SubscriptionsVO getSubscriptionByThirdSubscriptionId(String thirdSubscriptionId);

    /**
     * 获取当前支付方式所有订阅信息
     *
     * @param userId userId
     * @param payment payment
     * @return SubscriptionsVO
     */
    List<SubscriptionsVO> getSubscriptionsByPayment(String userId, Integer payment);

    /**
     * 获取前七天需要校验得订阅数据
     *
     * @param minusDays         minusDays
     * @param paymentMethodEnum paymentMethodEnum
     * @return EquityVerificationDTO
     */
    List<EquityVerificationDTO> getSubscriptionsByEndDate(LocalDateTime minusDays, PaymentMethodEnum paymentMethodEnum);

    int updateStatusByUserIdAndProductId(Integer status, String userId, String productId);

    Boolean judgeAiDiscount(String userId);
}
