package cn.kdan.cloud.pdf.office.payment.service.impl;

import cn.kdan.cloud.pdf.office.api.account.feign.UserApi;
import cn.kdan.cloud.pdf.office.api.account.feign.UserSubscriptionInfoApi;
import cn.kdan.cloud.pdf.office.api.email.bo.EmailSendBO;
import cn.kdan.cloud.pdf.office.api.email.feign.EmailApi;
import cn.kdan.cloud.pdf.office.api.payment.appstore.AppTransaction;
import cn.kdan.cloud.pdf.office.api.payment.constant.AppStoreAPIConstant;
import cn.kdan.cloud.pdf.office.api.payment.dto.*;
import cn.kdan.cloud.pdf.office.api.payment.enums.PaymentMethodEnum;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrdersVO;
import cn.kdan.cloud.pdf.office.api.product.feign.ProductApi;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductVO;
import cn.kdan.cloud.pdf.office.common.enums.EmailCodeTypeEnum;
import cn.kdan.cloud.pdf.office.common.enums.ExceptionEnum;
import cn.kdan.cloud.pdf.office.common.enums.payment.SubscriptionTypeEnum;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.utils.CommonUtils;
import cn.kdan.cloud.pdf.office.common.utils.MyDateUtils;
import cn.kdan.cloud.pdf.office.common.vo.UserInfoVO;
import cn.kdan.cloud.pdf.office.common.vo.UserVO;
import cn.kdan.cloud.pdf.office.payment.client.AppStoreClient;
import cn.kdan.cloud.pdf.office.payment.properties.AppStoreProperties;
import cn.kdan.cloud.pdf.office.payment.service.AppStoreService;
import cn.kdan.cloud.pdf.office.payment.service.OrderService;
import cn.kdan.cloud.pdf.office.payment.service.RestorePurchaseLogsService;
import cn.kdan.cloud.pdf.office.payment.service.SubscriptionsService;
import cn.kdan.cloud.pdf.office.payment.utils.TemplatesUtil;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.util.StreamUtils;
import org.springframework.web.client.RestTemplate;

import java.io.IOException;
import java.math.BigDecimal;
import java.nio.charset.StandardCharsets;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @author ComPDFKit-WPH 2023/6/29
 *
 * AppStoreServiceImpl
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class AppStoreServiceImpl implements AppStoreService {

    private final AppStoreClient appStoreClient;

    private final OrderService orderService;

    private final SubscriptionsService subscriptionsService;


    private final ProductApi productApi;

    private final UserApi userApi;


    private final EmailApi emailApi;
    private final AppStoreProperties properties;
    private static final RestTemplate restTemplate = new RestTemplate();

    @Value("${htmlToPdfUrl:http://139.196.160.101:3060/api/get-invoice}")
    private String htmlToPdfUrl;



    @Transactional
    @Override
    public UserInfoVO appStoreEquityVerification(EquityVerificationDTO equityVerificationDTO) {
        // 没有票据直接返回用户
        if (StringUtils.isEmpty(equityVerificationDTO.getTransactionId())) {
            UserInfoVO result = userApi.getInfoById(equityVerificationDTO.getUserId()).getResult();
            result.setDigestPassword(null);
            return result;
        }
        log.info("appStore 权益校验,用户id：{},请求体：{}",equityVerificationDTO.getUserId(),equityVerificationDTO);
        AppTransaction appTransaction = this.appStoreReceiptVerify(equityVerificationDTO.getTransactionId(),equityVerificationDTO.getApplePayProductId(),equityVerificationDTO.getAppBundleId());
        // 获取票据订单信息
        String transactionId = appTransaction.getTransactionId();
        String originalTransactionId = appTransaction.getOriginalTransactionId();
        BigDecimal price = BigDecimal.valueOf(appTransaction.getPrice()/1000);
        String currency = appTransaction.getCurrency();
        OrdersVO orderByThirdOrderNo = orderService.getOrderByThirdOrderNo(transactionId);
        if (null != orderByThirdOrderNo) {
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_TICKET_HAS_BEEN_CREATED);
        }
            // 没有绑定直接建立新的绑定关系
            log.info("新购买订单，票据{}，用户绑定{}", equityVerificationDTO.getTransactionId(), equityVerificationDTO.getUserId());
            // 新增新用户的订阅
            CreateSubscriptionDTO createSubscription = new CreateSubscriptionDTO();
            ProductVO product = productApi.getProductByCode(equityVerificationDTO.getProductCode()).getResult();
            String subscriptionId = CommonUtils.generateId();
            String orderId = cn.kdan.cloud.pdf.office.payment.utils.CommonUtils.generateRightsId((long) ((Math.random() * 9999 + 1) * 10000));
            String appId = product.getAppId();
            createSubscription.setId(subscriptionId);
            createSubscription.setUserId(equityVerificationDTO.getUserId());
            // 使用购买的订单号
            createSubscription.setOrderId(orderId);
            createSubscription.setAppId(appId);
            createSubscription.setPayment(PaymentMethodEnum.APPLE_PAY.getValue());
            createSubscription.setProductId(product.getId());
            createSubscription.setThirdSubscriptionId(originalTransactionId);
            UserVO userVO  = userApi.getById(equityVerificationDTO.getUserId()).getResult();
            createSubscription.setEmail(userVO.getEmail());
            createSubscription.setBuyerType(4);//2官网购买 3后台赠送 4appstore
            createSubscription.setSubscriptionType(SubscriptionTypeEnum.MEMBER.value());//如果是0则是试用
            createSubscription.setPaymentModel(product.getPaymentModel());
            createSubscription.setPrice(price);
            createSubscription.setPayTime(0);
            subscriptionsService.createSubscription(createSubscription);
            String readeNo = cn.kdan.cloud.pdf.office.payment.utils.CommonUtils.generateRightsId((long) ((Math.random() * 9999 + 1) * 10000));
           // 更新订单
            CreateOrderManualDTO orderManualDTO = CreateOrderManualDTO.builder()
                    .thirdTradeNo(originalTransactionId)
                    .thirdOrderNo(transactionId)
                    .price(price)
                    .reducedPrice(price)
                    .subscriptionId(subscriptionId)
                    .userId(equityVerificationDTO.getUserId())
                    .email(userVO.getEmail())
                    .id(orderId)
                    .paymentMethod(PaymentMethodEnum.APPLE_PAY)
                    .appId(appId)
                    .productId(product.getId())
                    .paymentModel(product.getPaymentModel())
                    .subscriptionType(1)
                    .payNumber(1)
                    .tradeNo(readeNo)
                    .invoiceNo(MyDateUtils.getTimeStamp() + (int) ((Math.random() * 9 + 1) * 1000))
                    .result(appTransaction.toString())
                    .currency(currency)
                    .build();
        orderService.createOrderManual(orderManualDTO);
        UserInfoVO result = userApi.getInfoById(equityVerificationDTO.getUserId()).getResult();
        result.setDigestPassword(null);
        sendBuyEmail(product, userVO,orderManualDTO);
        return result;
    }

    private void sendBuyEmail(ProductVO product, UserVO userVO,CreateOrderManualDTO orderManualDTO) {
        if(product.getCode().contains("upgrade")){
            EmailSendBO bo = new EmailSendBO();
            bo.setToEmail(userVO.getEmail());
            //设置事件
            bo.setUseEvent(EmailCodeTypeEnum.MEMBER_UPGRADE_SUCCESS.value());
            //设置title
            Map<String,String> titleMap = new HashMap<>();
            //设置内容
            Map<String,String> contentMap = new HashMap<>();
            contentMap.put("@plan@", product.getProductName());
            Date date = new Date();
            // 创建 SimpleDateFormat 对象，指定日期格式
            SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");
            // 格式化当前日期
            String formattedDate = sdf.format(date);
            contentMap.put("@date@",formattedDate);
            String price;
            BigDecimal renewPrice;
            if(orderManualDTO.getCurrency().equals("USD")){
                price = ObjectUtils.isNotEmpty(product.getDisplayPrice())? product.getDisplayPrice().toString() : product.getPrice().toString();
                renewPrice = product.getPrice();
            }else{
                price = ObjectUtils.isNotEmpty(product.getCnyDisplayPrice())? product.getCnyDisplayPrice().toString() : product.getCnyPrice().toString();
                renewPrice = product.getCnyPrice();
            }
            contentMap.put("@payPrice@", orderManualDTO.getCurrency()+price);
            contentMap.put("@renewPrice@",orderManualDTO.getCurrency()+price);

            bo.setSendTitleContent(titleMap);
            //设置内容
            bo.setSendContent(contentMap);
            String file = sendInvoice(userVO.getEmail(),formattedDate,orderManualDTO.getTradeNo(),orderManualDTO.getInvoiceNo(),renewPrice,orderManualDTO.getPrice(),orderManualDTO.getCurrency());
            bo.setFileUrl(file);
            bo.setFileName("invoice.pdf");
            emailApi.sendEmail(bo);
        }else{
            EmailSendBO bo = new EmailSendBO();
            bo.setToEmail(userVO.getEmail());
            //设置事件
            bo.setUseEvent(EmailCodeTypeEnum.MEMBER_BUY_SUCCESS.value());
            //设置title
            Map<String,String> titleMap = new HashMap<>();
            //设置内容
            Map<String,String> contentMap = new HashMap<>();
            contentMap.put("@plan@", product.getProductName());
            Date date = new Date();
            // 创建 SimpleDateFormat 对象，指定日期格式
            SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");
            // 格式化当前日期
            String formattedDate = sdf.format(date);
            contentMap.put("@date@",formattedDate);
            String price;
            BigDecimal renewPrice;
            if(orderManualDTO.getCurrency().equals("USD")){
                price = ObjectUtils.isNotEmpty(product.getDisplayPrice())? product.getDisplayPrice().toString() : product.getPrice().toString();
                renewPrice = product.getPrice();
            }else{
                price = ObjectUtils.isNotEmpty(product.getCnyDisplayPrice())? product.getCnyDisplayPrice().toString() : product.getCnyPrice().toString();
                renewPrice = product.getCnyPrice();
            }
            contentMap.put("@payPrice@", orderManualDTO.getCurrency()+price);
            bo.setSendTitleContent(titleMap);
            //设置内容
            bo.setSendContent(contentMap);
            String file = sendInvoice(userVO.getEmail(),formattedDate,orderManualDTO.getTradeNo(),orderManualDTO.getInvoiceNo(),renewPrice,orderManualDTO.getPrice(),orderManualDTO.getCurrency());
            bo.setFileUrl(file);
            bo.setFileName("invoice.pdf");
            emailApi.sendEmail(bo);
        }
    }

    //生成发票
    private String sendInvoice(String email, String purchasedDate,String tradeNo,String invoiceNumber,BigDecimal unitPrice,BigDecimal amount,String currency) {
        String invoiceHtml = null;
        try {
            invoiceHtml = new String(StreamUtils.copyToByteArray(this.getClass().getClassLoader().getResourceAsStream("templates/invoice_member.html")), StandardCharsets.UTF_8);
        } catch (IOException e) {
            log.error(e.getMessage());
        }
        Map<String, Object> map = new HashMap<>();
        map.put("Name", email);
        map.put("email", email);
        map.put("purchasedDate", purchasedDate);
        map.put("orderNumber",tradeNo);
        map.put("invoiceDate", purchasedDate);
        map.put("invoiceNumber", invoiceNumber);
        map.put("quantity", 1);
        map.put("unitPrice", unitPrice);
        BigDecimal discountAmount = unitPrice.subtract(amount);
        map.put("discount", discountAmount);
        map.put("amount", amount);
        map.put("total", amount);
        log.info("发票html内容填充map:{}", map);
        // 生成替换内容后的发票
        String htmlStr = TemplatesUtil.replaceStringUsingFreeMarker(invoiceHtml, map);
        if (!currency.equals("USD")) {
            htmlStr = htmlStr.replace("USD", currency);
        }
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_FORM_URLENCODED);
        // 封装表单数据
        MultiValueMap<String, String> paramMap = new LinkedMultiValueMap<>();
        paramMap.add("html", htmlStr);
        HttpEntity<MultiValueMap<String, String>> request = new HttpEntity<>(paramMap, headers);
        // 发送POST请求,html转pdf
        Map postForObjectMap = restTemplate.postForObject(htmlToPdfUrl, request, Map.class);
        String invoiceFileUrl = postForObjectMap.get("url").toString();
        log.info("发票文件地址：{}", invoiceFileUrl);
        return invoiceFileUrl;
    }

    /**
     * 票据验证
     *
     * @return List<InApp>
     */
    private AppTransaction appStoreReceiptVerify(String transactionId, String applePayProductId,String appBundleId) {
        // 验证票据信息
        AppTransaction transaction = appStoreClient.fetchFirstAppTransaction(transactionId,appBundleId);
        // 如果主环境交易记录为null，尝试获取沙箱环境交易记录
        if (Objects.isNull(transaction) && "false".equals(properties.getIsSandbox())) {
            transaction = appStoreClient.fetchSandboxFirstAppTransaction(transactionId, appBundleId);
        }

        // 如果最终transaction仍为null，抛出验证失败异常
        if (Objects.isNull(transaction)) {
            log.error("验证苹果交易失败，transactionId：{}", transactionId);
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_APP_STORE_TRANSACTION_ID_VALIDATION_FAILED);
        }
        log.info("苹果票据信息：{}", transaction.toString());
        // 检查支付状态等
        if (transaction.getProductId().equals(applePayProductId)) {
            Date expiresDateMs = transaction.getExpiresDate();
            String inAppOwnershipType = transaction.getInAppOwnershipType();
            // 校验是否是支付成功的订单
            if (!AppStoreAPIConstant.INAPPOWNERSHIPTYPE_PURCHASED.equals(inAppOwnershipType)) {
                log.error("验证苹果交易失败，inAppOwnershipType：{},AppTransaction:{}",inAppOwnershipType,transaction.toString());
                throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_APP_STORE_TRANSACTION_ID_VALIDATION_FAILED);
            }
            // 校验订阅到期时间是否在当前时间之后
            if (expiresDateMs!=null&&expiresDateMs.before(new Date())) {
                log.error( "验证苹果交易失败,票据已过期，AppTransaction:{} " ,transaction);
                throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_APP_STORE_TRANSACTION_ID_VALIDATION_FAILED);
            }
            if(!transaction.getProductId().equals(applePayProductId)){
                log.error( "验证苹果交易失败,产品id不一致,AppTransaction:{} " ,transaction);
                throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_APP_STORE_TRANSACTION_ID_VALIDATION_FAILED);
            }
        }
        return transaction;
    }

}
