package cn.kdan.cloud.pdf.office.payment.service.impl;

import cn.kdan.cloud.pdf.office.api.payment.constant.OrderConstant;
import cn.kdan.cloud.pdf.office.api.payment.dto.PaddleWebhookDTO;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrdersVO;
import cn.kdan.cloud.pdf.office.payment.service.OrderService;
import cn.kdan.cloud.pdf.office.payment.service.PaddleWebhookService;
import cn.kdan.cloud.pdf.office.payment.service.WebhookService;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author ComPDFKit-WPH 2023/1/30
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class PaddleWebhookServiceImpl implements PaddleWebhookService {

    private final OrderService ordersService;


    private final WebhookService webhookService;




    @Override
    public void subscriptionPaymentSucceeded(PaddleWebhookDTO paddleWebhookDTO) {
        log.info("paddle 订阅的支付订单已成功 paddle订单id：{}", paddleWebhookDTO.getCheckoutId());
        String thirdTradeNo = paddleWebhookDTO.getPassthrough();
        List<OrdersVO> ordersVOS = ordersService.getOrderByTradeNo(thirdTradeNo);
        if (!CollectionUtils.isEmpty(ordersVOS) && ordersVOS.size() == 1 && ordersVOS.get(0).getStatus().equals(OrderConstant.CREATED)) {
            log.info("首期订阅业务处理开始 paypal订单id：{}", thirdTradeNo);
            webhookService.handlingUserSubscriptionActivation(paddleWebhookDTO.getBalanceGross(), thirdTradeNo,
                    paddleWebhookDTO.getOrderId(),  ordersVOS.get(0), paddleWebhookDTO.getSubscriptionId());
        } else {
            String paddleOrderId = paddleWebhookDTO.getOrderId();
            OrdersVO ordersVO = ordersService.getOrderByThirdOrderNo(paddleOrderId);
            if (ObjectUtils.isNotEmpty(ordersVO) && OrderConstant.COMPLETED.equals(ordersVO.getStatus())) {
                return;
            }
            //paddle第一次使用get Passthrough paddleWebhookDTO.getSubscriptionId()
            webhookService.handleSubsequentAutomaticDeduction(paddleWebhookDTO.getBalanceGross(),
                    paddleWebhookDTO.getSubscriptionId(), paddleWebhookDTO.getOrderId(), ordersVOS.get(0), 1, null,null );
        }
    }

    @Override
    public void subscriptionCancel(PaddleWebhookDTO paddleWebhookDTO) {
        String thirdSubscriptionId = paddleWebhookDTO.getSubscriptionId();
        log.info("订阅取消续订，paddle订阅id：{}", thirdSubscriptionId);
        webhookService.handleUnsubscribeFollowUpActions(thirdSubscriptionId);
    }
}
