package cn.kdan.cloud.pdf.office.payment.service.impl;

import cn.kdan.cloud.pdf.office.api.payment.constant.OrderConstant;
import cn.kdan.cloud.pdf.office.api.payment.dto.*;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrdersVO;
import cn.kdan.cloud.pdf.office.payment.service.*;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.List;

/**
 * @author ComPDFKit-WPH 2023/1/30
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class PayPalWebhookServiceImpl implements PayPalWebhookService {

    private final OrderService ordersService;


    private final WebhookService webhookService;


    @Transactional
    @Override
    public void subscriptionPaymentSucceeded(PaypalWebhookMonitorDTO paypalWebhookMonitorDTO) {
        String thirdTradeNo = paypalWebhookMonitorDTO.getResource().getBilling_agreement_id();
        log.info("paypal 订阅的支付订单已成功 开始处理业务 paypal订单id：{}",thirdTradeNo);
        List<OrdersVO> ordersVOS = ordersService.getOrderByTradeNo(thirdTradeNo);
        // 判断是否是第一期
        // 如果该订阅的订单只有一笔，并且该笔订单的状态是已创建,则说明是第一期扣款的事件回调
        // 如果是第一期的话，根据价格，因为价格为0的话不会扣款就不会进入这个事件，所以进入这件事件的一定是价格不为0的
        // 对于没有试用的计划的第一期，则由本事件处理，订阅激活的时候不做处理，则修改订单状态，用户状态和发邮件
        BigDecimal price = new BigDecimal(paypalWebhookMonitorDTO.getResource().getAmount().getTotal());
        if(!CollectionUtils.isEmpty(ordersVOS)&& ordersVOS.size() == 1 && ordersVOS.get(0).getStatus().equals(OrderConstant.CREATED)){
            String thirdOrderId = paypalWebhookMonitorDTO.getResource().getId();
            log.info("首期订阅业务处理开始 paypal订单id：{}",thirdTradeNo);
            webhookService.handlingUserSubscriptionActivation(price,thirdTradeNo,thirdOrderId,ordersVOS.get(0),thirdTradeNo);
        }else{
            log.info("自动订阅业务处理开始（非首期） paypal订单id：{}",thirdTradeNo);
            // 如果不是第一期的话
            // 则直接增加订单，并且增加订阅时间
            // 因为paypal第一次创建订阅的时候没有标识
            String thirdOrderId = paypalWebhookMonitorDTO.getResource().getId();
            OrdersVO ordersVO = ordersService.getOrderByThirdOrderNo(thirdOrderId);
            if (ObjectUtils.isNotEmpty(ordersVO) && OrderConstant.COMPLETED.equals(ordersVO.getStatus())) {
                return;
            }
            OrdersVO orderByTradeNo = ordersVOS.get(0);
            webhookService.handleSubsequentAutomaticDeduction(price, thirdTradeNo,
                    thirdOrderId, orderByTradeNo, null, null,null );
        }
    }

    @Override
    public void subscriptionCancel(PaypalWebhookMonitorDTO paypalWebhookMonitorDTO) {
        String thirdSubscriptionId = paypalWebhookMonitorDTO.getResource().getId();
        log.info("订阅取消续订，paypal订阅id：{}", thirdSubscriptionId);
        // 根据第三放订阅id查询对应的用户id和产品id
        webhookService.handleUnsubscribeFollowUpActions(thirdSubscriptionId);
    }

    @Override
    public void subscriptionPayFail(PaypalWebhookMonitorDTO paypalWebhookMonitorDTO) {
        log.info("订阅续订支付失败，paypal订阅id：{}", paypalWebhookMonitorDTO.getResource().getId());
    }


}
