package cn.kdan.cloud.pdf.office.payment.service.impl;

import cn.kdan.cloud.pdf.office.api.account.feign.UserApi;
import cn.kdan.cloud.pdf.office.api.account.feign.UserSubscriptionInfoApi;
import cn.kdan.cloud.pdf.office.api.account.vo.UserSubscriptionInfo;
import cn.kdan.cloud.pdf.office.api.email.bo.EmailSendBO;
import cn.kdan.cloud.pdf.office.api.email.feign.EmailApi;
import cn.kdan.cloud.pdf.office.api.payment.bo.CreateSubscriptionsResultPaypalBO;
import cn.kdan.cloud.pdf.office.api.payment.bo.OrderSucceededBO;
import cn.kdan.cloud.pdf.office.api.payment.constant.OrderConstant;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreateOrderManualDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreateSubscriptionDTO;
import cn.kdan.cloud.pdf.office.api.payment.enums.PaymentMethodEnum;
import cn.kdan.cloud.pdf.office.api.payment.paddle.SubscriptionWithUser;
import cn.kdan.cloud.pdf.office.api.payment.paypal.transaction.PaypalTransaction;
import cn.kdan.cloud.pdf.office.api.payment.paypal.transaction.TransactionsItem;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrderUpdateVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrdersVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.SubscriptionsVO;
import cn.kdan.cloud.pdf.office.api.product.feign.ProductApi;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductVO;
import cn.kdan.cloud.pdf.office.common.dto.UserRegisterDTO;
import cn.kdan.cloud.pdf.office.common.enums.EmailCodeTypeEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.AccountSourceEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.PDFOfficeUserSubscriptionPayTypeEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.PlatformTypeEnum;
import cn.kdan.cloud.pdf.office.common.enums.payment.SubscriptionTypeEnum;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import cn.kdan.cloud.pdf.office.common.utils.CommonUtils;
import cn.kdan.cloud.pdf.office.common.utils.MyDateUtils;
import cn.kdan.cloud.pdf.office.common.vo.UserVO;
import cn.kdan.cloud.pdf.office.payment.client.PaddleClient;
import cn.kdan.cloud.pdf.office.payment.client.PaypalClient;
import cn.kdan.cloud.pdf.office.payment.enums.PaypalTransactionStatusEnum;
import cn.kdan.cloud.pdf.office.payment.enums.PaypalWebhookEventEnum;
import cn.kdan.cloud.pdf.office.payment.service.OrderService;
import cn.kdan.cloud.pdf.office.payment.service.SubscriptionsService;
import cn.kdan.cloud.pdf.office.payment.service.WebhookService;
import cn.kdan.cloud.pdf.office.payment.utils.TemplatesUtil;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Service;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.util.StreamUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.client.RestTemplate;

import java.io.IOException;
import java.math.BigDecimal;
import java.nio.charset.StandardCharsets;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @author ComPDFKit-WPH 2023/1/30
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class WebhookServiceImpl implements WebhookService {

    private final OrderService ordersService;

    private final SubscriptionsService subscriptionsService;

    private final UserApi userApi;

    private final UserSubscriptionInfoApi userSubscriptionInfoApi;
    private final ProductApi productApi;

    private static final RestTemplate restTemplate = new RestTemplate();

    @Value("${htmlToPdfUrl:http://139.196.160.101:3060/api/get-invoice}")
    private String htmlToPdfUrl;

    private final EmailApi emailApi;

    private final RedissonClient redissonClient;

    private final PaypalClient paypalClient;

    private final PaddleClient paddleClient;

    @Override
    public void handleSubsequentAutomaticDeduction(BigDecimal price, String thirdTradeNo, String thirdOrderId, OrdersVO orderByTradeNo, Integer payTime, String result, BigDecimal oldPrice) {
        log.info("自动订阅业务处理开始（非首期）订单id：{}", thirdOrderId);
        String subId = CommonUtils.generateId();
        String orderId = cn.kdan.cloud.pdf.office.payment.utils.CommonUtils.generateRightsId((long) ((Math.random() * 9999 + 1) * 10000));;
        // 创建新订单
        CreateOrderManualDTO createOrderManual = new CreateOrderManualDTO();
        BeanUtils.copyProperties(orderByTradeNo, createOrderManual);
        createOrderManual.setPaymentMethod(PaymentMethodEnum.fromValue(orderByTradeNo.getPayment()));
        // 设置实际扣费价格
        if(ObjectUtils.isNotEmpty(oldPrice)){
            createOrderManual.setPrice(oldPrice);
        }else {
            createOrderManual.setPrice(price);
        }
        createOrderManual.setReducedPrice(price);
        if(!StringUtils.isEmpty(orderByTradeNo.getCurrency())){
            createOrderManual.setCurrency(orderByTradeNo.getCurrency());
        }
        if(!StringUtils.isEmpty(result)){
            createOrderManual.setResult(result);
        }
        // 设置第三方passthrough
        createOrderManual.setThirdTradeNo(orderByTradeNo.getThirdTradeNo());
        // 设置第三方回调id
        createOrderManual.setThirdOrderNo(thirdOrderId);
        createOrderManual.setSubscriptionId(subId);
        createOrderManual.setEmail(orderByTradeNo.getEmail());
        String readeNo = cn.kdan.cloud.pdf.office.payment.utils.CommonUtils.generateRightsId((long) ((Math.random() * 9999 + 1) * 10000));
        createOrderManual.setTradeNo(readeNo);
        createOrderManual.setInvoiceNo(MyDateUtils.getTimeStamp() + (int) ((Math.random() * 9 + 1) * 1000));
        createOrderManual.setId(orderId);
        createOrderManual.setPaymentModel(2);
        createOrderManual.setPayNumber(1);
        createOrderManual.setUserId(orderByTradeNo.getUserId());
        ordersService.createOrderManual(createOrderManual);
        //创建订阅时间并且，发送订阅消息，并且更新用户模块的信息
        CreateSubscriptionDTO subscriptionManual = new CreateSubscriptionDTO();
        subscriptionManual.setId(subId);
        subscriptionManual.setProductId(orderByTradeNo.getProductId());
        subscriptionManual.setThirdSubscriptionId(thirdTradeNo);
        subscriptionManual.setUserId(orderByTradeNo.getUserId());
        subscriptionManual.setAppId(orderByTradeNo.getAppId());
        subscriptionManual.setEmail(userApi.getById(subscriptionManual.getUserId()).getResult().getEmail());
        //正式会员
        subscriptionManual.setSubscriptionType(SubscriptionTypeEnum.MEMBER.value());
        subscriptionManual.setPaymentModel(2);
        subscriptionManual.setOrderId(orderId);
        subscriptionManual.setPrice(price);
        subscriptionManual.setPayment(orderByTradeNo.getPayment());
        if(ObjectUtils.isNotEmpty(payTime)){
            subscriptionManual.setPayTime(payTime);
        }
        subscriptionsService.createSubscription(subscriptionManual);
    }
    void handleEmailSendBO (EmailCodeTypeEnum action, EmailSendBO bo) {
        //设置事件
        bo.setUseEvent(action.value());
        //设置title
        Map<String,String> titleMap = new HashMap<>();
        bo.setSendTitleContent(titleMap);
    }

    @Override
    public OrdersVO getOrderWithUpdate(String orderId) {
        OrdersVO ordersVO = ordersService.getOrderById(orderId);
        //如果订单还没有完成，则去第三方查询订单状态，更新订单和订阅
        if(ordersVO.getStatus().equals(OrderConstant.CREATED)){
            PaymentMethodEnum paymentMethodEnum = PaymentMethodEnum.fromValue(ordersVO.getPayment());
            String thirdSubId = ordersVO.getThirdTradeNo();
            switch (paymentMethodEnum) {
                case PAYPAL:
                    //第三方查询订单状态，更新订单和订阅
                    //根据第三方订阅id查询
                    String lockKey = PaypalWebhookEventEnum.SALE_COMPLETED.value().concat(thirdSubId) + "-RedissonLock";
                    // 获取 RLock 对象
                    RLock rLock = redissonClient.getLock(lockKey);
                    try {
                        // 上锁
                        boolean resultLock = rLock.tryLock(30, 10, TimeUnit.SECONDS);
                        // 业务逻辑开始
                        if (resultLock) {
                            log.info("上锁成功");
                            List<OrdersVO> ordersVOS = ordersService.getOrderByTradeNo(thirdSubId);
                            //查询用户20分钟内交易记录，如果成功设置交易id
                            LocalDateTime dateTime = LocalDateTime.now();
                            DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'");
                            String startTime = ordersVO.getCreatedAt().format(formatter);
                            String endTime = dateTime.format(formatter);
                            PaypalTransaction result =  paypalClient.getListTransactionsForSubscription(thirdSubId,startTime,endTime);
                            List<TransactionsItem> transactions = result.getTransactions();
                            if(CollectionUtils.isNotEmpty(transactions)&&transactions.get(0).getStatus().equals(PaypalTransactionStatusEnum.COMPLETED.value())){
                                //因为是首次订阅所以只有一笔交易，所以直接拿第一笔就行
                                TransactionsItem transactionsItem = transactions.get(0);
                                BigDecimal price = new BigDecimal(transactionsItem.getAmountWithBreakdown().getGrossAmount().getValue());
                                this.handlingUserSubscriptionActivation(price,thirdSubId,transactionsItem.getId(),ordersVOS.get(0),thirdSubId);
                                return ordersService.getOrderById(orderId);
                            }
                        }
                    } catch(InterruptedException e){
                        log.info("上锁失败");
                        e.printStackTrace();
                    } finally{
                        if (rLock.isLocked() && rLock.isHeldByCurrentThread()) {
                            rLock.unlock();
                        }
                    }

                    break;
                default:
                    break;
            }
        }
        return ordersVO;
    }


    @Override
    public void handlingUserSubscriptionActivation(BigDecimal price, String thirdTradeNo, String thirdOrderNo, OrdersVO order, String thirdSubsId) {
        UserVO userVO = userApi.getByAppAccount(order.getEmail(), PlatformTypeEnum.WEBSITE.value().toString()).getResult();
        if(ObjectUtils.isEmpty(userVO)){
            //创建账号
            userVO = register(order, userVO);
        }
        // 添加订阅信息
        String subId = CommonUtils.generateId();
        OrderSucceededBO orderSucceeded = OrderSucceededBO.builder()
                .thirdTradeNo(thirdTradeNo)
                .price(price)
                .subscriptionId(subId)
                .thirdOrderNo(thirdOrderNo)
                .userId(userVO.getId())
                .build();
        if (PaymentMethodEnum.APPLE_PAY.getValue().equals(order.getPayment())) {
            orderSucceeded.setOrderId(order.getId());
        }
        // 修改订单状态
        OrderUpdateVO orderUpdate = ordersService.updateOrderSucceeded(orderSucceeded);
        CreateSubscriptionDTO createSubscription = new CreateSubscriptionDTO();
        createSubscription.setId(subId);
        createSubscription.setUserId(orderUpdate.getUserId());
        createSubscription.setOrderId(orderUpdate.getOrderId());
        createSubscription.setAppId(orderUpdate.getAppId());
        createSubscription.setPayment(order.getPayment());
        createSubscription.setProductId(orderUpdate.getProductId());
        if(order.getPayment().equals(PaymentMethodEnum.PADDLE.getValue())){
            createSubscription.setThirdSubscriptionId(thirdSubsId);
        }else{
            createSubscription.setThirdSubscriptionId(thirdTradeNo);
        }
        createSubscription.setEmail(userVO.getEmail());
        if (PaymentMethodEnum.APPLE_PAY.getValue().equals(order.getPayment())) {
            //2官网购买 3后台赠送 4appstore
            createSubscription.setBuyerType(4);
        }
        //如果是0则是试用
        createSubscription.setSubscriptionType(SubscriptionTypeEnum.MEMBER.value());
        subscriptionsService.createSubscription(createSubscription);
        if(orderUpdate.getAppId().equals("16")){
            // 用户信息
            // 查询订单信息获取产品id
            SubscriptionsVO subscriptionsVO = subscriptionsService.getSubscriptionBySubscriptionId(createSubscription.getId());
            //发送邮件给用户
            EmailSendBO bo = new EmailSendBO();
            bo.setToEmail(userVO.getEmail());
            Map<String,String> contentMap = new HashMap<>();
            ProductVO productVO = productApi.getProduct(order.getProductId()).getResult();
            contentMap.put("@order_number@",order.getTradeNo());
            contentMap.put("@product@", productVO.getProductName());
            contentMap.put("@subscription@","yearly plan");
            DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd hh:mm");
            contentMap.put("@purchase_date@",dateFormat.format(new Date()));
            contentMap.put("@price@",price.toString());
            contentMap.put("@expiration_date@",dateFormat.format(subscriptionsVO.getEndDate()));
            bo.setSendContent(contentMap);

            //设置内容 @order_number@ @product@ @subscription@ @purchase_date@  @price@ @exiration_date@
            handleEmailSendBO(EmailCodeTypeEnum.SUBSCRIPTION_SUCCESS,bo);
            String invoiceHtml = null;
            try {
                invoiceHtml = new String(StreamUtils.copyToByteArray(this.getClass().getClassLoader().getResourceAsStream("templates/Invoice.html")), StandardCharsets.UTF_8);
            } catch (IOException e) {
                log.error(e.getMessage());
            }
//        log.info("发票html内容填充前: {}",invoiceHtml);
            // 填充字段
            Map<String, Object> map = new HashMap<>();
            map.put("email", userVO.getEmail());
            map.put("orderNumber", order.getTradeNo());
            map.put("purchasedDate", dateFormat.format(new Date()));
            map.put("invoiceDate", dateFormat.format(new Date()));
            map.put("invoiceNumber", order.getInvoiceNo());
            map.put("productName", productVO.getProductName());
            map.put("unitPrice", productVO.getPrice());
            BigDecimal discount = productVO.getPrice().subtract(price);
            map.put("discount", discount);
            map.put("amount", price);
            map.put("total", price);
            String htmlStr = TemplatesUtil.replaceStringUsingFreeMarker(invoiceHtml, map);
            // 生成发票,  C端发票操作
            // 设置请求header
            log.info("发票html内容填充map: {}",map.toString());
            HttpHeaders headers = new HttpHeaders();
            headers.setContentType(MediaType.APPLICATION_FORM_URLENCODED);
            // 封装表单数据
            MultiValueMap<String, String> paramMap = new LinkedMultiValueMap<>();
            paramMap.add("html", htmlStr);
            HttpEntity<MultiValueMap<String, String>> request = new HttpEntity<>(paramMap, headers);
            // 发送POST请求
            Map postForObjectMap = restTemplate.postForObject(htmlToPdfUrl, request, Map.class);
            String invoiceFileUrl = postForObjectMap.get("url").toString();
            bo.setFileUrl(invoiceFileUrl);
            bo.setFileName("invoice.pdf");
            emailApi.sendEmail(bo);
        }
    }

    /**
     * 给未注册的购买用户创建账号
     * @param order order
     * @param userVO userVO
     * @return UserVO
     */
    private UserVO register(OrdersVO order, UserVO userVO) {
        UserRegisterDTO userRegisterDTO = new UserRegisterDTO();
        userRegisterDTO.setAppId(order.getAppId());
        String pwd = CommonUtils.generateRandomStringId(10);
        userRegisterDTO.setPassword(pwd);
        userRegisterDTO.setPlatformType(PlatformTypeEnum.WEBSITE.value().toString());
        userRegisterDTO.setUsername(order.getEmail());
        ProductVO productVO = productApi.getProduct(order.getProductId()).getResult();
        userRegisterDTO.setModel(productVO.getModel());
        userRegisterDTO.setAccountSource(AccountSourceEnum.PAY.value());
        ResultMap<Boolean> resultMap = userApi.register(userRegisterDTO);
        if(resultMap.getResult()){
            userVO = userApi.getByAppAccount(order.getEmail(), PlatformTypeEnum.WEBSITE.value().toString()).getResult();
            //发送邮件给用户
            EmailSendBO bo = new EmailSendBO();
            bo.setToEmail(userVO.getEmail());
            Map<String,String> contentMap = new HashMap<>();
            contentMap.put("@product_name@",productVO.getProductName());
            contentMap.put("@plan_name@"," yearly plan");
            contentMap.put("@account@", userVO.getEmail());
            contentMap.put("@password@",pwd);
            bo.setSendContent(contentMap);
            //设置内容 @product_name@ @plan_name@ @account@ @password@
            handleEmailSendBO(EmailCodeTypeEnum.SUBSCRIPTION_SUCCESS_WITH_REGISTER,bo);
            emailApi.sendEmail(bo);
        }
        return userVO;
    }


    /**
     * 取消自动订阅的后续操作
     * @param thirdSubscriptionId 第三方订阅id
     */
    @Override
    public void handleUnsubscribeFollowUpActions(String thirdSubscriptionId) {
        SubscriptionsVO subscription = subscriptionsService.getSubscriptionByThirdSubscriptionId(thirdSubscriptionId);
        if(ObjectUtils.isEmpty(subscription)){
            log.error("第三方订阅id{}未绑定",thirdSubscriptionId);
            return;
        }
        String userId = subscription.getUserId();
        String productId = subscription.getProductId();
        // 根据用户id和产品id修改用户订阅状态 不是自动订阅
        log.info("userId: {} productId:{}", userId, productId);
        UserSubscriptionInfo userSubscriptionInfo = userSubscriptionInfoApi.getByUserIdAndProductId(userId, productId).getResult();
        log.info("userSubscriptionInfo id: {} ", userSubscriptionInfo.getId());
        SubscriptionsVO latestSubscription = subscriptionsService.getLatestSubscriptionForActualPayment(userId, productId);
        //用户最新的实际订阅
        log.info("LatestSubscriptionForActualPayment id: {} ", latestSubscription.getId());
        PaymentMethodEnum paymentMethodEnum = PaymentMethodEnum.fromValue(latestSubscription.getPayment());
        switch (paymentMethodEnum) {
            case PADDLE:
                List<SubscriptionWithUser> list = paddleClient.getSubscriptionInfo(latestSubscription.getThirdSubscriptionId());
                //如果查出来有记录说明处于订阅状态，则不需要修改状态
                if (!CollectionUtils.isEmpty(list)) {
                    return;
                }
                break;
            case PAYPAL:
                CreateSubscriptionsResultPaypalBO subscriptionsResultPaypalBO = paypalClient.getSubscriptionInfo(latestSubscription.getThirdSubscriptionId());
                //如果订阅状态是活跃的，则不需要修改状态
                if ("ACTIVE".equals(subscriptionsResultPaypalBO.getStatus())) {
                    return;
                }
                break;
            default:
                return;
        }
        UserSubscriptionInfo info = new UserSubscriptionInfo();
        info.setUserId(userId);
        info.setProductId(productId);
        info.setPayType(PDFOfficeUserSubscriptionPayTypeEnum.NON_AUTO_SUBSCRIPTION.value());
        info.setId(userSubscriptionInfo.getId());
        userSubscriptionInfoApi.update(info);
    }

    @Override
    public void handleMemberUnsubscribeFollowUpActions(String thirdSubscriptionId) {
        SubscriptionsVO subscription = subscriptionsService.getSubscriptionByThirdSubscriptionId(thirdSubscriptionId);
        if(ObjectUtils.isEmpty(subscription)){
            log.error("第三方订阅id{}未绑定",thirdSubscriptionId);
            return;
        }
        if(StringUtils.isEmpty(subscription.getId())){
            log.error("第三方订阅id{}未绑定",thirdSubscriptionId);
            return;
        }
        String userId = subscription.getUserId();
        Integer platform = productApi.getProduct(subscription.getProductId()).getResult().getPlatform();
        // 根据用户id和产品id修改用户订阅状态 不是自动订阅
        log.info("userId: {} productId:{}", userId, platform);
        UserSubscriptionInfo userSubscriptionInfo = userSubscriptionInfoApi.getByUserIdAndPlatform(userId, platform).getResult();
        log.info("userSubscriptionInfo id: {} ", userSubscriptionInfo.getId());
        UserSubscriptionInfo info = new UserSubscriptionInfo();
        info.setUserId(userId);
        info.setPlatform(platform);
        info.setPayType(PDFOfficeUserSubscriptionPayTypeEnum.NON_AUTO_SUBSCRIPTION.value());
        info.setId(userSubscriptionInfo.getId());
        userSubscriptionInfoApi.update(info);
    }
}
