package cn.kdan.cloud.pdf.office.website.controller;

import cn.kdan.cloud.pdf.office.api.account.feign.AIApi;
import cn.kdan.cloud.pdf.office.api.account.feign.UserApi;
import cn.kdan.cloud.pdf.office.api.account.feign.UserSubscriptionInfoApi;
import cn.kdan.cloud.pdf.office.api.account.vo.AiRecord;
import cn.kdan.cloud.pdf.office.api.account.vo.UserSubscriptionInfo;
import cn.kdan.cloud.pdf.office.common.constant.CommonConstant;
import cn.kdan.cloud.pdf.office.common.enums.AIActionEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.PDFOfficeUserSubscriptionStatusEnum;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.pojo.CustomUserDetails;
import cn.kdan.cloud.pdf.office.common.utils.CommonUtils;
import cn.kdan.cloud.pdf.office.common.utils.JsonUtils;
import cn.kdan.cloud.pdf.office.common.vo.UserSubscriptionInfoVO;
import cn.kdan.cloud.pdf.office.website.dto.ai.AIParam;
import cn.kdan.cloud.pdf.office.website.dto.ai.TranslateResult;
import cn.kdan.cloud.pdf.office.website.service.impl.AIServiceImpl;
import cn.kdan.cloud.pdf.office.website.utils.SecurityUtils;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.security.Principal;
import java.util.*;

/**
 * ai模块
 */
@RestController
@RequestMapping("/api")
@RequiredArgsConstructor
public class AIController {
    @Autowired
    private UserApi userApi;
    @Autowired
    private UserSubscriptionInfoApi userSubscriptionInfoApi;
    @Autowired
    private AIServiceImpl aiService;
    @Autowired
    private AIApi aiApi;

    /**
     * 错别字纠正
     * @param param param
     * @param principal
     * @return
     */
    @Transactional
    @RequestMapping("/correct-typos")
    public Object correctTypos (@RequestBody AIParam param,Principal principal){
        UserSubscriptionInfoVO vo = getAiSubscriptionInfo(principal);
        if(!checkPoint(vo, 1)){
            TranslateResult<Map<String,Integer>> result = new TranslateResult<Map<String,Integer>>();
            result.setCode("500");
            result.setMessage("run out");
            Map<String,Integer> map = new HashMap<>();
            map.put("credit",1);
            result.setData(map);
            return result;
        }
        try {
            Object object = aiService.correctTypos(param);
            decPoint(vo, 1);
            aiRecord(param.getContent(), vo.getUserId(), object,1,AIActionEnum.CORRECTION);
            return object;
        } catch (Exception e) {
            TranslateResult<String> result = new TranslateResult<String>();
            result.setCode("500");
            result.setMessage("run out");
            return result;
        }
    }

    private void aiRecord(String content, String userId, Object object, Integer point,AIActionEnum actionEnum) {
        AiRecord aiRecord = new AiRecord();
        aiRecord.setFunction(actionEnum.getCode());
        aiRecord.setId(CommonUtils.generateId());
        aiRecord.setUserId(userId);
        if(!StringUtils.isEmpty(content)){
            aiRecord.setContent(content);
        }
        aiRecord.setResult(JsonUtils.getJsonString(object));
        aiRecord.setCredit(point);
        aiRecord.setCreatedAt(new Date());
        aiApi.insert(aiRecord);
    }

    /**
     * 提取摘要
     * @param param param
     * @return
     */
    @Transactional
    @RequestMapping("/extract-file-summary")
    public Object extractFileSummary (@RequestParam MultipartFile file, Principal principal){
        UserSubscriptionInfoVO vo = getAiSubscriptionInfo(principal);
        if(!checkPoint(vo, 1)){
            TranslateResult<Map<String,Integer>> result = new TranslateResult<Map<String,Integer>>();
            result.setCode("500");
            result.setMessage("run out");
            Map<String,Integer> map = new HashMap<>();
            map.put("credit",1);
            result.setData(map);
            return result;
        }
        try {
            AIParam param = new AIParam();
            param.setFile(file);
            Object object = aiService.extractFileSummary(param);
            decPoint(vo, 1);
            aiRecord(param.getContent(), vo.getUserId(), object,1,AIActionEnum.EXTRACT_SUMMARY);
            return object;
        } catch (Exception e) {
            TranslateResult<String> result = new TranslateResult<String>();
            result.setCode("500");
            result.setMessage("run out");
            return result;
        }
    }

    /**
     * 重写
     * @param param param
     * @return
     */
    @Transactional
    @RequestMapping("/rewrite")
    public Object rewrite (@RequestBody AIParam param,Principal principal){
        UserSubscriptionInfoVO vo = getAiSubscriptionInfo(principal);
        if(!checkPoint(vo, 1)){
            TranslateResult<Map<String,Integer>> result = new TranslateResult<Map<String,Integer>>();
            result.setCode("500");
            result.setMessage("run out");
            Map<String,Integer> map = new HashMap<>();
            map.put("credit",1);
            return result;
        }
        try {
            Object object = aiService.rewrite(param);
            decPoint(vo, 1);
            aiRecord(param.getContent(), vo.getUserId(), object,1,AIActionEnum.REWRITE);
            return object;
        } catch (Exception e) {
            TranslateResult<String> result = new TranslateResult<String>();
            result.setCode("500");
            result.setMessage("run out");
            return result;
        }
    }
    public UserSubscriptionInfoVO getAiSubscriptionInfo(Principal principal) {
        String userId = SecurityUtils.getUserId();
        List<UserSubscriptionInfoVO> voList =  userSubscriptionInfoApi.getByUserIdAndAppId(userId,"1").getResult();
        Optional<UserSubscriptionInfoVO> activeUserSubscriptionInfoVO = voList.stream()
                .filter(vo -> {
                    // Check if 'isAi' is not empty
                    return !ObjectUtils.isEmpty(vo.getIsAi()) ;
                })
                .findFirst(); // Get the first matching VO
        // Check if
        if (!activeUserSubscriptionInfoVO.isPresent()) {
            throw new RuntimeException("No AI subscription found for user: " + userId);
        }
        // Get the first matching VO from the list (should be the only one) and return it as the active subscription info.
        UserSubscriptionInfoVO userSubscriptionInfoVO = activeUserSubscriptionInfoVO.get();
        return userSubscriptionInfoVO;
    }

    public boolean checkPoint(UserSubscriptionInfoVO infoVO, Integer point){
        //如果point不够返回false
        if(infoVO.getPoint()<point){
            return false;
        }
        return true;
    }

    public void decPoint(UserSubscriptionInfoVO infoVO,Integer point){
        UserSubscriptionInfo userSubscriptionInfo = new UserSubscriptionInfo();
        userSubscriptionInfo.setId(infoVO.getId());
        userSubscriptionInfo.setPoint(infoVO.getPoint() - point);
        userSubscriptionInfoApi.update(userSubscriptionInfo);
    }


}
