package cn.kdan.cloud.pdf.office.website.controller;

import cn.kdan.cloud.pdf.office.api.account.feign.UserApi;
import cn.kdan.cloud.pdf.office.api.product.feign.PrizeApi;
import cn.kdan.cloud.pdf.office.api.product.vo.PrizeVO;
import cn.kdan.cloud.pdf.office.common.constant.CommonConstant;
import cn.kdan.cloud.pdf.office.common.enums.ExceptionEnum;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.pojo.CustomUserDetails;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import cn.kdan.cloud.pdf.office.website.service.LotteryService;
import cn.kdan.cloud.pdf.office.website.utils.SecurityUtils;
import com.alibaba.nacos.common.utils.CollectionUtils;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.security.Principal;
import java.util.List;

/**
 * 活动模块
 */
@RestController
@RequestMapping("/activity")
@RequiredArgsConstructor
public class ActivityController {
    private final PrizeApi prizeApi;
    private final LotteryService lotteryService;

    /**
     * 黑五判断用户有没有抽过奖
     * @return
     */
    @PostMapping("/canDoLottery")
    public ResultMap<Boolean> canDoLottery() {
        String userId = SecurityUtils.getUserId();
        //判断用户有没有抽过奖
        PrizeVO prizeVO = new PrizeVO();
        prizeVO.setUserId(userId);
        //1是黑五抽奖
        prizeVO.setActivityType("1");
        List<PrizeVO> list= prizeApi.list(prizeVO).getResult();
        if(CollectionUtils.isEmpty(list)){
            return ResultMap.success(true);
        }
        return ResultMap.success(false);
    }

    /**
     * 抽奖
     * @param principal
     * @return
     */
    @PostMapping("/lottery")
    public ResultMap<Integer> lottery(Principal principal) {
        String userId = SecurityUtils.getUserId();
        //判断用户有没有抽过奖
        PrizeVO prizeVO = new PrizeVO();
        prizeVO.setUserId(userId);
        //1是黑五抽奖
        prizeVO.setActivityType("1");
        List<PrizeVO> list= prizeApi.list(prizeVO).getResult();
        if(!CollectionUtils.isEmpty(list)){
           throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_USER_ALREADY_LOTTERY);
        }
        return ResultMap.success(lotteryService.lottery(SecurityUtils.getUserId()));
    }

    private String getUserId(Principal principal) {
        if (principal instanceof OAuth2Authentication) {
            return ((CustomUserDetails) (((OAuth2Authentication) principal).getPrincipal())).getId();
        }
        throw new BackendRuntimeException(CommonConstant.EXCEPTION_MSG_LOGIN_USER_NOT_FOUND);
    }
}
