package cn.kdan.cloud.pdf.office.website.controller;

import cn.kdan.cloud.pdf.office.api.payment.dto.CancelSubscriptionDTO;
import cn.kdan.cloud.pdf.office.api.payment.vo.SubscriptionSchemeResultVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.SubscriptionSchemeVO;
import cn.kdan.cloud.pdf.office.api.product.enums.PlatformEnum;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductVO;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import cn.kdan.cloud.pdf.office.website.dto.IsSubscriptionDTO;
import cn.kdan.cloud.pdf.office.website.service.PaymentService;
import cn.kdan.cloud.pdf.office.website.service.ProductInfoService;
import com.baomidou.mybatisplus.core.metadata.IPage;
import lombok.RequiredArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotNull;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;

/**
 * @author ComPDFKit-WPH 2023/2/9
 * <p>
 * PDF OFFICE app端 Controller
 */
@RestController
@RequestMapping("/app")
@RequiredArgsConstructor
public class PdfOfficeAppController {

    private final PaymentService paymentService;

    private final ProductInfoService productInfoService;

    /**
     * 获取订单记录
     *
     * @return ResultMap
     */
    @GetMapping("/getOrderRecord")
    public ResultMap<Map<String,SubscriptionSchemeVO>> getOrderRecord(@RequestParam String appId) {
        return paymentService.getSubscriptionSchemeInfo(appId);
    }

    /**
     * 获取订阅结束时间
     *
     * @param platform 平台
     * @return LocalDateTime
     */
    @GetMapping("/getEndDate")
    public ResultMap<LocalDateTime> getEndDate(@RequestParam Integer platform) {
        return ResultMap.success(paymentService.getSubscriptionEndDate(platform));
    }

    /**
     * 获取新用户专用活动
     *
     * @param appId appId
     * @return 新用户专用活动
     */
    @GetMapping("/getNewUserProduct")
    public ResultMap<List<ProductVO>> getNewUserProduct(@Validated @NotNull(message = "appId不能为空") @RequestParam String appId) {
        return productInfoService.getNewUserProduct(appId);
    }

    /**
     * 获取所有产品
     *
     * @param appId appId
     * @return 获取所有产品
     */
    @GetMapping("/getProducts")
    public ResultMap<IPage<ProductVO>> getProductList(@Validated @NotNull(message = "appId不能为空") @RequestParam String appId,
                                                      @RequestParam Long size,
                                                      @RequestParam Long page) {
        return productInfoService.getProductList(size, page, appId);
    }


    /**
     * 判断是否有订阅
     *
     * @param isSubscriptionDTO isSubscriptionDTO
     * @return Boolean true: 折扣，false：原价
     */
    @PostMapping("/isSubscription")
    public ResultMap<Boolean> isSubscription(@RequestBody @Validated IsSubscriptionDTO isSubscriptionDTO) {
        return ResultMap.success(paymentService.isSubscription(isSubscriptionDTO));
    }

}
