package cn.kdan.cloud.pdf.office.website.controller;

import cn.kdan.cloud.pdf.office.api.product.vo.ListingProductVO;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import cn.kdan.cloud.pdf.office.website.service.ProductInfoService;
import lombok.RequiredArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import java.util.List;
import java.util.Map;

/**
 * @author ComPDFKit-Bob 2024/10/21
 * <p>
 * PDF OFFICE web端 Controller
 */
@RestController
@RequestMapping("/web")
@RequiredArgsConstructor
public class PdfOfficeWebController {

    private final ProductInfoService productInfoService;

    /**
     * 获取上架产品，返回前端购买
     * @param isEducation
     * @param platformId 1 官网，2 dmg，3 Mac store免费版，4e xe，5 Mac store付费版，6i OS免费版，7 iOS付费版，8 Android
     */
    @GetMapping("/getListingProducts")
    public ResultMap<List<ListingProductVO>> getListingProducts(@Validated @NotNull(message = "是否教育优惠不能为空") @RequestParam("isEducation") Integer isEducation,
                                                                @Validated @NotNull(message = "平台id不能为空") @RequestParam("platformId") String platformId){
        return productInfoService.getListingProducts(isEducation, platformId);
    }

    /**
     * 获取购买商品价格，验证用户教育购买权限
     */
    @GetMapping("/getProductPriceForBuy")
    public ResultMap<ListingProductVO> getProductPriceForBuy(@Validated @NotNull(message = "产品id不能为空") @RequestParam("productId") String productId,
                                                             @RequestParam(value = "userId", required = false) String userId,
                                                             @Validated @NotNull(message = "是否教育优惠不能为空") @RequestParam("isEducation") Integer isEducation) {
        return ResultMap.success(productInfoService.getProductPriceForBuy(productId, userId, isEducation));
    }

    @GetMapping("/checkEducation")
    public ResultMap<Boolean> checkEducation(@Validated @NotNull(message = "邮箱不能为空") @RequestParam("email") String email) {
        return ResultMap.success(productInfoService.checkEducation(email));
    }

    /**
     * 获取批量购买价格
     */
    @GetMapping("/getBatchProductPrice")
    public ResultMap<Map<String, String>> getBatchProductPrice(@Validated @NotNull(message = "产品id不能为空") @RequestParam("productId") String productId,
                                                                          @Validated @NotNull(message = "购买数量不能为空") @Min(2) @RequestParam("num") Integer num) {
        return ResultMap.success(productInfoService.getBatchProductPrice(productId, num));
    }

    /**
     * 验证优惠券、奖品是否可用，返回优惠后价格
     */
    @GetMapping("/checkCoupon")
    public ResultMap<ListingProductVO> checkCoupon(@Validated @NotNull(message = "产品id不能为空") @RequestParam("productId") String productId,
                                                   @RequestParam(required = false, name = "userId") String userId,
                                                   @NotNull(message = "券码不能为空") @RequestParam("code") String code) {
        return productInfoService.checkCoupon(productId,userId, code);
    }
}
