package cn.kdan.cloud.pdf.office.website.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.kdan.cloud.pdf.office.api.account.feign.UserApi;
import cn.kdan.cloud.pdf.office.api.payment.feign.OrderApi;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrdersVO;
import cn.kdan.cloud.pdf.office.api.product.feign.ProductApi;
import cn.kdan.cloud.pdf.office.api.product.vo.ListingProductVO;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductVO;
import cn.kdan.cloud.pdf.office.common.enums.ExceptionEnum;
import cn.kdan.cloud.pdf.office.common.enums.product.ProductEducationPriceEnum;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import cn.kdan.cloud.pdf.office.common.vo.UserInfoVO;
import cn.kdan.cloud.pdf.office.common.vo.UserSubscriptionInfoVO;
import cn.kdan.cloud.pdf.office.common.vo.UserVO;
import cn.kdan.cloud.pdf.office.website.service.ProductInfoService;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestParam;

import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author ComPDFKit-WPH 2023/2/6
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class ProductInfoServiceImpl implements ProductInfoService {

    private final ProductApi productApi;

    private final UserApi userAPi;

    private final OrderApi orderApi;

    @Override
    public ResultMap<List<ProductVO>> getNewUserProduct(String appId) {
        return productApi.getNewUserProduct(appId);
    }


    @Override
    public ResultMap<IPage<ProductVO>> getProductList(Long size, Long page, String appId) {
        return productApi.getProductList(size, page, appId);
    }

    @Override
    public ResultMap<List<ListingProductVO>> getListingProducts(Integer isEducation, String platformId) {
        return productApi.getListingProducts(isEducation, platformId);
    }

    @Override
    public ListingProductVO getProductPriceForBuy(String productId, String userId, Integer isEducation) {
        // 查询购买的商品信息
        List<ListingProductVO> productVOs = productApi.getProductsByIds(Collections.singletonList(productId)).getResult();
        if (CollectionUtils.isEmpty(productVOs)) {
            log.error("商品不存在，id：{}", productId);
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_PRODUCT_NOT_EXIST);
        }
        ListingProductVO listingProductVO = productVOs.get(0);
        // 查询用户+所有订单
        if (StringUtils.isEmpty(userId)) {
            return listingProductVO;
        }
        UserInfoVO userVO = userAPi.getMemberInfoById(userId).getResult();
        if (null == userVO) {
            log.error("用户不存在，id：{}", userId);
            return listingProductVO;
        }
        List<UserSubscriptionInfoVO> subscriptionInfoList = userVO.getSubscriptionInfoList();
        // 买断的产品 或者 没有购买过的用户直接返回价格
        if (CollectionUtils.isEmpty(subscriptionInfoList) || listingProductVO.getPaymentModel() == 2) {
            return listingProductVO;
        }
        // 订阅产品，判断用户购买情况和是否买断升级订阅
        List<UserSubscriptionInfoVO> userSubscriptionInfoVOList = subscriptionInfoList.stream().filter(Objects::nonNull).filter(s -> s.getPlatform() != 0 && s.getStatus() != 2 && s.getStatus() != 4).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(userSubscriptionInfoVOList)) {
            return listingProductVO;
        }
        // 筛选订单购买的产品中有没有vip订阅类型
        boolean hasVip = false;
        UserSubscriptionInfoVO vipSubscriptionInfoVO = null;
        boolean hasSVip = false;
        UserSubscriptionInfoVO sVipSubscriptionInfoVO = null;
        // 买断版  平台数
        int permanentPlatform = 0;
        for (UserSubscriptionInfoVO userSubscriptionInfoVO : userSubscriptionInfoVOList) {
            // 买断的产品，判断是否多平台
            if (userSubscriptionInfoVO.getPaymentModel().equals("2") && StringUtils.isNotEmpty(userSubscriptionInfoVO.getPlatforms())) {
                int count = 0;
                if (userSubscriptionInfoVO.getPlatforms().contains("Mac")) count++;
                if (userSubscriptionInfoVO.getPlatforms().contains("Windows")) count++;
                if (userSubscriptionInfoVO.getPlatforms().contains("iOS")) count++;
                if (userSubscriptionInfoVO.getPlatforms().contains("Android")) count++;
                if (count == 1) {
                    permanentPlatform = 1;
                } else if (count > 1) {
                    permanentPlatform = 2;
                }
            } else {
                if (StringUtils.isNotEmpty(userSubscriptionInfoVO.getLevels())&&userSubscriptionInfoVO.getLevels().contains("2")) {
                    hasVip = true;
                    vipSubscriptionInfoVO = userSubscriptionInfoVO;
                } else if (StringUtils.isNotEmpty(userSubscriptionInfoVO.getLevels())&&userSubscriptionInfoVO.getLevels().contains("3")) {
                    hasSVip = true;
                    sVipSubscriptionInfoVO = userSubscriptionInfoVO;
                }
            }
        }
        // 购买svip，判断是否买断升级订阅（买且只买了永久版，购买高级版订阅）
        if (!StringUtils.isEmpty(listingProductVO.getLevels())&&listingProductVO.getLevels().contains("3")) {
            // 有vip无法购买svip
            if (hasVip) {
                throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_USER_CANNOT_BUY_PREMIUM);
            }
            // 有svip购买svip
            if (hasSVip) {
                // 判断用户是否还在订阅中,
                if (subscriptionInfoList.stream().filter(Objects::nonNull)
                        .anyMatch(userSubscriptionInfoVO -> userSubscriptionInfoVO.getPlatform().equals(2) && userSubscriptionInfoVO.getPayType() == 1)) {
                    throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_USER_ALREADY_SUBSCRIBED);
                }
                if (!sVipSubscriptionInfoVO.getProductId().equals(productId)) {
                    throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_USER_ALREADY_SUBSCRIBED);
                }
                // svip原价提前续费
                return listingProductVO;
            }
            // 买断升级订阅价格
            if (permanentPlatform != 0) {
                if (null != isEducation && isEducation == 1) {
                    listingProductVO.setUpgradePrice(new BigDecimal("25.99"));
                    listingProductVO.setCnyUpgradePrice(new BigDecimal("169"));
                } else if (permanentPlatform == 1) {
                    listingProductVO.setUpgradePrice(new BigDecimal("49.99"));
                    listingProductVO.setCnyUpgradePrice(new BigDecimal("328"));
                } else if (permanentPlatform == 2) {
                    listingProductVO.setUpgradePrice(new BigDecimal("39.99"));
                    listingProductVO.setCnyUpgradePrice(new BigDecimal("259"));
                }
                return listingProductVO;
            }
        }
        // 购买vip
        if (!StringUtils.isEmpty(listingProductVO.getLevels())&&listingProductVO.getLevels().contains("2")) {
            // vip提前续费
            if (hasVip) {
                // 判断用户是否还在订阅中,
                if (subscriptionInfoList.stream().filter(Objects::nonNull)
                        .anyMatch(userSubscriptionInfoVO -> userSubscriptionInfoVO.getPlatform().equals(2) && userSubscriptionInfoVO.getPayType() == 1)) {
                    throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_USER_ALREADY_SUBSCRIBED);
                }
                if (!vipSubscriptionInfoVO.getProductId().equals(productId)) {
                    throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_USER_ALREADY_SUBSCRIBED);
                }
                return listingProductVO;
            }
            // svip无法购买vip
            if (hasSVip) {
                throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_USER_CANNOT_BUY_VIP);
            }
            // 买断无法购买vip
            if (permanentPlatform != 0) {
                throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_USER_CANNOT_UPGRADE_VIP);
            }
        }
        return listingProductVO;
    }

    @Override
    public Boolean checkEducation(String email) {
        Integer checkEmail = productApi.checkEmail(email).getResult();
        if (checkEmail == 0) {
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_USER_IS_NOT_EDUCATION_USER);
        }
        return true;
    }

    /**
     * 批量获取产品价格
     * @return
     */
    @Override
    public Map<String, String> getBatchProductPrice(String productId, Integer num) {
        ProductVO productVO = productApi.getProduct(productId).getResult();
        if (productVO == null) {
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_PRODUCT_NOT_EXIST);
        }
        // 目前只支持标准买断可以批量购买的阶梯价格 advanced-permanent
        BigDecimal batchPrice, totalPrice, cnyBatchPrice, cnyTotalPrice, discountPrice, cnyDiscountPrice;
        if ("advanced-permanent".equals(productVO.getCode())) {
            // 标准买断
            // 2-5	10% OFF	$107.99/个  ¥701.93/个
            //6-20	20% OFF	$95.99/个   ¥633.93/个
            //21-50	30% OFF	$83.99/个   ¥545.93/个
            //50+	        $71.99/个   ¥467.93/个
            if (num <= 1) {
                batchPrice = productVO.getPrice();
                totalPrice = productVO.getPrice();
                cnyBatchPrice = productVO.getCnyPrice();
                cnyTotalPrice = productVO.getCnyPrice();
            } else if (num <= 5) {
                batchPrice = new BigDecimal("107.99");
                totalPrice = batchPrice.multiply(new BigDecimal(num));
                cnyBatchPrice = new BigDecimal("701.93");
                cnyTotalPrice = cnyBatchPrice.multiply(new BigDecimal(num));
            } else if (num <= 20) {
                batchPrice = new BigDecimal("95.99");
                totalPrice = batchPrice.multiply(new BigDecimal(num));
                cnyBatchPrice = new BigDecimal("633.93");
                cnyTotalPrice = cnyBatchPrice.multiply(new BigDecimal(num));
            } else if (num <= 50) {
                batchPrice = new BigDecimal("83.99");
                totalPrice = batchPrice.multiply(new BigDecimal(num));
                cnyBatchPrice = new BigDecimal("545.93");
                cnyTotalPrice = cnyBatchPrice.multiply(new BigDecimal(num));
            } else {
                batchPrice = new BigDecimal("71.99");
                totalPrice = batchPrice.multiply(new BigDecimal(num));
                cnyBatchPrice = new BigDecimal("467.93");
                cnyTotalPrice = cnyBatchPrice.multiply(new BigDecimal(num));
            }
        } else if ("advanced-permanent-blackFive".equals(productVO.getCode())) {
            // 黑五买断  优惠价×数量
            batchPrice = productVO.getDisplayPrice();
            totalPrice = productVO.getDisplayPrice().multiply(new BigDecimal(num));
            cnyBatchPrice = productVO.getCnyDisplayPrice();
            cnyTotalPrice = productVO.getCnyDisplayPrice().multiply(new BigDecimal(num));
        } else {
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_PRODUCT_NOT_SUPPORT_BATCH);
        }
        discountPrice = productVO.getPrice().multiply(BigDecimal.valueOf(num)).subtract(totalPrice);
        cnyDiscountPrice = productVO.getCnyPrice().multiply(BigDecimal.valueOf(num)).subtract(cnyTotalPrice);
        Map<String, String> map = new HashMap<>();
        map.put("productId", productId);
        map.put("num", String.valueOf(num));
        map.put("price", productVO.getPrice().toEngineeringString());
        map.put("cnyPrice", productVO.getCnyPrice().toEngineeringString());
        map.put("batchPrice", batchPrice.toEngineeringString());
        map.put("totalPrice", totalPrice.toEngineeringString());
        map.put("discountPrice", discountPrice.toEngineeringString());
        map.put("cnyBatchPrice", cnyBatchPrice.toEngineeringString());
        map.put("cnyTotalPrice", cnyTotalPrice.toEngineeringString());
        map.put("cnyDiscountPrice", cnyDiscountPrice.toEngineeringString());
        return map;

    }

    @Override
    public ResultMap<ListingProductVO> checkCoupon(String productId, String userId, String code) {
        return productApi.checkCoupon(productId, userId, code);
    }

    @Override
    public ResultMap<Boolean> updateCouponState(String code, Integer state, String orderId) {
        return productApi.updateCouponState(code, state, orderId);
    }
}
