package cn.kdan.cloud.pdf.office.product.controller;

import cn.kdan.cloud.pdf.office.api.product.dto.ProductListDTO;
import cn.kdan.cloud.pdf.office.api.product.vo.ListingProductVO;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductPriceVo;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductVO;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import cn.kdan.cloud.pdf.office.product.entity.Product;
import cn.kdan.cloud.pdf.office.product.service.ProductService;
import com.baomidou.mybatisplus.core.metadata.IPage;
import lombok.RequiredArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.util.List;
import java.util.Map;

/**
 * @author ComPDFKit-WPH 2023/2/1
 */
@RestController
@RequestMapping("/product")
@RequiredArgsConstructor
public class ProductController {

    private final ProductService productService;

    @GetMapping("/getProductPrice")
    public ResultMap<ProductPriceVo> getProductPrice(@Validated @NotNull(message = "产品id不能为空") @RequestParam("productId") String productId,
                                                     @NotNull(message = "userId不能为空") @RequestParam("userId") String userId) {

        return ResultMap.success(productService.getProductPrice(productId, userId));
    }


    @GetMapping("/getProduct")
    public ResultMap<ProductVO> getProduct(@Validated @NotNull(message = "产品id不能为空") @RequestParam("productId") String productId) {
        return ResultMap.success(productService.getProduct(productId));
    }
    /**
     * 获取产品（带第三方支付id）
     * @param productId
     * @param payment
     * @param isDiscount
     * @return
     */
    @GetMapping("/getProductForPay")
    public ResultMap<ProductVO> getProductForPay(@Validated @NotNull(message = "产品id不能为空") @RequestParam("productId") String productId,
                                                 @Validated @NotNull(message = "支付方式不能为空") @RequestParam("payment") Integer payment,
                                                 @Validated @NotNull(message = "是否首年优惠不能为空") @RequestParam("isDiscount") Integer isDiscount) {
        return ResultMap.success(productService.getProductForPay(productId,payment,isDiscount ));
    }


    /**
     * 获取产品列表
     *
     * @param size  size
     * @param page  page
     * @param appId appId
     * @return ProductVO
     */
    @GetMapping("/getProductList")
    public ResultMap<IPage<ProductVO>> getProductList(@RequestParam Long size, @RequestParam Long page,
                                                      @RequestParam String appId) {
        ProductListDTO productListDTO = new ProductListDTO();
        productListDTO.setSize(size);
        productListDTO.setPage(page);
        productListDTO.setAppId(appId);
        return ResultMap.success(productService.page(productListDTO));
    }

    @GetMapping("/getProductByCode")
    public ResultMap<ProductVO> getProductByCode(@RequestParam("code") String productCode) {
        return ResultMap.success(productService.getProductByCode(productCode));
    }


    /**
     * 获取新用户专用活动
     *
     * @param appId appId
     * @return 新用户专用活动
     */
    @GetMapping("/getNewUserProduct")
    public ResultMap<List<ProductVO>> getNewUserProduct(@RequestParam String appId) {
        return ResultMap.success(productService.getNewUserProduct(appId));
    }

    @GetMapping("/getProductByAppIdAndModel")
    public ResultMap<ProductVO> getProductByAppIdAndModel(@RequestParam String appId,@RequestParam Integer platform) {
        return ResultMap.success(productService.getProductByAppIdAndModel(appId,platform));
    }

    @GetMapping("/getProductByThirdPayId")
    public ResultMap<ProductVO> getProductByThirdPayId(@RequestParam("thirdPayId") String thirdPayId){
        return ResultMap.success(productService.getProductByThirdPayId(thirdPayId));
    }

    /**
     * 获取上架产品，返回前端购买
     * @param isEducation
     * @param platformId 1 官网，2 dmg，3 Mac store免费版，4e xe，5 Mac store付费版，6i OS免费版，7 iOS付费版，8 Android
     */
    @GetMapping("/getListingProducts")
    public ResultMap<List<ListingProductVO>> getListingProducts(@Validated @NotNull(message = "是否教育优惠不能为空") @RequestParam("isEducation") Integer isEducation,
                                                                @Validated @NotNull(message = "平台id不能为空") @RequestParam("platformId") String platformId){
        return ResultMap.success(productService.getListingProducts(isEducation, platformId));
    }

    @GetMapping("/getProductsByIds")
    public ResultMap<List<ListingProductVO>> getProductsByIds(@RequestParam(value = "productIds", required = false) List<String> productIds) {
        return ResultMap.success(productService.getProductsByIds(productIds));
    }

    /**
     * 验证用户邮箱教育优惠资格
     * @param email 用户邮箱
     * @return
     */
    @GetMapping("/checkEmail")
    public ResultMap<Integer> checkEmail(@RequestParam("email") String email) {
        return ResultMap.success(productService.checkEmail(email));
    }

/*    *//**
     * 获取购买商品价格，验证用户教育购买权限
     *//*
    @GetMapping("/getProductPriceForBuy")
    public ResultMap<ListingProductVO> getProductPriceForBuy(@Validated @NotNull(message = "产品id不能为空") @RequestParam("productId") String productId,
                                                           @Validated @RequestParam("userId") String userId,
                                                           @Validated @NotNull(message = "是否教育优惠不能为空") @RequestParam("isEducation") Integer isEducation) {
        return ResultMap.success(productService.getProductPriceForBuy(productId,userId, isEducation));
    }*/

    /**
     * 获取批量购买价格
     */
    @GetMapping("/getBatchProductPrice")
    public ResultMap<BigDecimal> getBatchProductPrice(@Validated @NotNull(message = "产品id不能为空") @RequestParam("productId") String productId,
                                                      @Validated @NotNull(message = "购买数量不能为空") @Min(2) @RequestParam("num") Integer num) {
        return ResultMap.success(productService.getBatchProductPrice(productId, num));
    }

    /**
     * 验证优惠券、奖品是否可用，返回优惠后价格
     */
    @GetMapping("/checkCoupon")
    public ResultMap<ListingProductVO> checkCoupon(@Validated @NotNull(message = "产品id不能为空") @RequestParam("productId") String productId,
                                                 @RequestParam(required = false, name = "userId") String userId,
                                                 @NotNull(message = "券码不能为空") @RequestParam("code") String code) {
        return ResultMap.success(productService.checkCoupon(productId,userId, code));
    }

    @PostMapping("/updateCouponState")
    public ResultMap<Boolean> updateCouponState(@Validated @NotNull(message = "券码不能为空") @RequestParam("code") String code,
                                                @Validated @NotNull(message = "状态不能为空") @RequestParam("state") Integer state,
                                                @Validated @NotNull(message = "订单id不能为空") @RequestParam("orderId") String orderId) {
        return ResultMap.success(productService.updateCouponState(code, state, orderId));
    }

    @PostMapping("/refundCouponByOrderId")
    ResultMap<Boolean> refundCouponByOrderId(@Validated @NotNull(message = "订单id不能为空") @RequestParam("orderId") String orderId) {
        return ResultMap.success(productService.refundCouponByOrderId(orderId));
    }
}
