package cn.kdan.cloud.pdf.office.sso.controller;

import cn.kdan.cloud.pdf.office.api.account.feign.UserApi;
import cn.kdan.cloud.pdf.office.api.email.bo.EmailSendBO;
import cn.kdan.cloud.pdf.office.api.email.feign.EmailApi;
import cn.kdan.cloud.pdf.office.api.product.feign.PrizeApi;
import cn.kdan.cloud.pdf.office.api.product.vo.PrizeVO;
import cn.kdan.cloud.pdf.office.common.constant.CommonConstant;
import cn.kdan.cloud.pdf.office.common.enums.EmailCodeTypeEnum;
import cn.kdan.cloud.pdf.office.common.enums.ExceptionEnum;
import cn.kdan.cloud.pdf.office.common.enums.ValidStatusEnum;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.pojo.CustomUserDetails;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import cn.kdan.cloud.pdf.office.common.vo.SimpleUserSubscriptionInfoVO;
import cn.kdan.cloud.pdf.office.common.vo.UserInfoVO;
import cn.kdan.cloud.pdf.office.common.vo.UserVO;
import cn.kdan.cloud.pdf.office.sso.pojo.AccountRightVO;
import cn.kdan.cloud.pdf.office.sso.service.AuthService;
import cn.kdan.cloud.pdf.office.sso.utils.RSAUtils;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.security.oauth2.provider.authentication.OAuth2AuthenticationDetails;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.security.Principal;
import java.time.*;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

/**
 * 个人中心模块
 */
@RestController
@RequestMapping("/userCenter")
public class UserCenterController {

    private final RSAUtils rsaUtils = new RSAUtils();
    @Autowired
    private UserApi userApi;
    @Autowired
    private EmailApi emailApi;
    @Autowired
    private AuthService authService;
    @Autowired
    private PrizeApi prizeApi;

    public static String convertToLowerCase(String input) {
        if (input == null || input.isEmpty()) {
            return ""; // 如果输入为空，返回空字符串
        }

        // 分割字符串
        String[] platforms = input.split(",");

        // 将平台名称转换为小写
        StringBuilder lowerCasePlatforms = new StringBuilder();
        for (int i = 0; i < platforms.length; i++) {
            lowerCasePlatforms.append(platforms[i].toLowerCase()); // 转换为小写
            if (i < platforms.length - 1) {
                lowerCasePlatforms.append(","); // 添加逗号分隔符
            }
        }

        return lowerCasePlatforms.toString(); // 返回结果字符串
    }



    /**
     * 获取用户个人信息
     *
     * @return info
     */
    @GetMapping("/accountRight")
    public ResultMap<AccountRightVO> info(Principal principal) {
        String userId = getUserId(principal);
        // 获取用户的基本信息
        UserInfoVO user = userApi.getMemberInfoById(userId).getResult();
        AccountRightVO result = new AccountRightVO();
        List<SimpleUserSubscriptionInfoVO> resultList = new ArrayList<>();
        AtomicInteger totalAiPoints = new AtomicInteger(); // 用于累积AI用户的点数
        AtomicBoolean isUpgrade = new AtomicBoolean(false);
        user.getSubscriptionInfoList().stream().forEach(item -> {
            item.setPlatforms(convertToLowerCase(item.getPlatforms()));
            if(item.getCode().contains("upgrade")&&item.getCode().contains("subscription")){
                isUpgrade.set(true);
            }
            // 没有购买显示当前时间
            if (ObjectUtils.isEmpty(item.getEndDate())) {
                item.setEndDate(new Date());
            }
            // 设置点数，如果为空则默认0
            if (ObjectUtils.isEmpty(item.getPoint())) {
                item.setPoint(0);
            }

            // 默认不是AI
            item.setIsAi(StringUtils.isEmpty(item.getIsAi()) ? "0" : item.getIsAi());

            if ("0".equals(item.getIsAi())) { // 非AI用户
                SimpleUserSubscriptionInfoVO vo = new SimpleUserSubscriptionInfoVO();
                BeanUtils.copyProperties(item, vo);
                resultList.add(vo); // 将非AI用户添加到结果列表
            } else { // AI用户
                totalAiPoints.addAndGet(item.getPoint()); // 累积AI用户的点数
            }
        });

        if (isUpgrade.get()) {
            resultList.removeIf(info -> Objects.equals(info.getPlatforms(), "3")); // 通过 removeIf 来移除 level 为 0 的记录
        }
        if (resultList.size() > 1) {
            resultList.removeIf(info -> Objects.equals(info.getLevels(), "1")); // 通过 removeIf 来移除 level 为 0 的记录
        }
        result.setVipList(resultList);
        result.setAiPoint(totalAiPoints.get());
        //Todo
        PrizeVO prizeVO = new PrizeVO();
        prizeVO.setUserId(userId);
        prizeVO.setValidFlag(Integer.valueOf(ValidStatusEnum.VALID.value()));
        List<PrizeVO> list = prizeApi.list(prizeVO).getResult();

        // 获取今天的日期
        LocalDate today = LocalDate.now();

         // 筛选 endDate 晚于今天的 PrizeVO
       // 筛选 endDate 晚于今天的 PrizeVO
        List<PrizeVO> filteredList = list.stream()
                .filter(prize -> {
                    Date endDate = prize.getEndDate();

                    LocalDate endLocalDate = endDate.toInstant().atZone(ZoneId.systemDefault()).toLocalDate();
                    return endLocalDate.isAfter(today);
                })
                .sorted((prize1, prize2) -> {
                    BigDecimal price1 = prize1.getUsdPrice();
                    BigDecimal price2 = prize2.getUsdPrice();

                    // 处理 null 值，null 排在最后，按价格降序排序
                    if (price1 == null && price2 == null) {
                        return 0;
                    } else if (price1 == null) {
                        return 1; // 如果 price1 是 null，则排在后面
                    } else if (price2 == null) {
                        return -1; // 如果 price2 是 null，则排在后面
                    } else {
                        // 正常情况下比较 BigDecimal，降序排序
                        return price2.compareTo(price1);
                    }
                })
                .collect(Collectors.toList());

        result.setGiftList(filteredList);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, result);
    }


    /**
     * 修改密码
     *
     * @param oldPwd    旧密码
     * @param firstPwd  新密码
     * @param secondPwd 确认密码
     * @return
     */
    @PostMapping("/updatePassword")
    public ResultMap<Boolean> updatePassword(@RequestParam String oldPwd, @RequestParam String firstPwd, @RequestParam String secondPwd, Principal principal) {
        String userId = getUserId(principal);
        UserVO userVO = userApi.getById(userId).getResult();
        if (!rsaUtils.decrypt(userVO.getDigestPassword()).equals(oldPwd)) {
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_PASSWORD_ERROR);
        }
        //如果两次密码不一致
        if (!firstPwd.equals(secondPwd)) {
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_PASSWORD_NOT_INCONSISTENT);
        }
        UserVO userVo = new UserVO();
        userVo.setId(userId);
        userVo.setUpdatedAt(new Date());
        userVo.setDigestPassword(rsaUtils.encrypt(firstPwd));
        userApi.updateUser(userVo);
        authService.logOutAll(userVO);
        handleEmailSendBO(EmailCodeTypeEnum.MEMBER_UPDATE_PASSWORD_SUCCESS, userVo.getEmail());
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, Boolean.TRUE);
    }

    /**
     * 设置密码
     *
     * @param firstPwd  第一次密码
     * @param secondPwd 确认密码
     * @return
     */
    @PostMapping("/setPassword")
    public ResultMap<Boolean> setPassword(@RequestParam String firstPwd, @RequestParam String secondPwd, Principal principal) {
        String userId = getUserId(principal);
        //如果两次密码不一致
        if (!firstPwd.equals(secondPwd)) {
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_PASSWORD_NOT_INCONSISTENT);
        }
        UserVO userVo = new UserVO();
        userVo.setId(userId);
        userVo.setUpdatedAt(new Date());
        userVo.setDigestPassword(rsaUtils.encrypt(firstPwd));
        userApi.updateUser(userVo);
        handleEmailSendBO(EmailCodeTypeEnum.MEMBER_UPDATE_PASSWORD_SUCCESS, userVo.getEmail());
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, Boolean.TRUE);
    }

    void handleEmailSendBO (EmailCodeTypeEnum action, String email) {
        EmailSendBO bo = new EmailSendBO();
        bo.setToEmail(email);
        //设置事件
        bo.setUseEvent(action.value());
        //设置title
        Map<String,String> titleMap = new HashMap<>();
        //设置内容
        Map<String,String> contentMap = new HashMap<>();
        // 获取当前的时间，设置为 GMT 时区
        ZonedDateTime now = ZonedDateTime.now(ZoneOffset.UTC);
        // 定义目标日期时间格式
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy/MM/dd HH:mm '(GMT)'");
        // 格式化当前时间
        String formattedTime = now.format(formatter);
        contentMap.put("@time@",formattedTime);
        contentMap.put("@email@",email);
        bo.setSendTitleContent(titleMap);
        //设置内容
        bo.setSendContent(contentMap);
        emailApi.sendEmail(bo);
    }

    /**
     * 个人信息修改（名，姓氏，地区，住址）
     * @param firstName 名
     * @param lastName 姓
     * @param address 住址
     * @param area 地区
     */
    @PostMapping("/updateInfo")
    public ResultMap<Boolean> updateInfo(@RequestParam String firstName, @RequestParam String lastName,
                                         @RequestParam String address, @RequestParam String area, Principal principal) {
        String userId = getUserId(principal);
        UserVO userVo = new UserVO();
        userVo.setId(userId);
        userVo.setUpdatedAt(new Date());
        userVo.setFirstName(firstName);
        userVo.setLastName(lastName);
        userVo.setArea(area);
        userVo.setAddress(address);
        userApi.updateUser(userVo);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, Boolean.TRUE);
    }

    /**
     * 修改邮箱
     * @param email 邮箱
     * @param code 验证码
     * @param appId id （1）
     * @return
     */
    @PostMapping("/updateEmail")
    public ResultMap<Boolean> updateEmail(@RequestParam String email, @RequestParam String code, @RequestParam String appId, Principal principal) {
        String userId = getUserId(principal);
        authService.checkEmailCodeValid(EmailCodeTypeEnum.MEMBER_UPDATE_EMAIL, email, code, appId);

        UserVO userVO = userApi.getById(userId).getResult();
        UserVO userVo = new UserVO();
        userVo.setId(userId);
        userVo.setUpdatedAt(new Date());
        userVo.setEmail(email);
        userApi.updateUser(userVo);
        authService.logOutAll(userVO);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, Boolean.TRUE);
    }

    private String getUserId(Principal principal) {
        if (principal instanceof OAuth2Authentication) {
            return ((CustomUserDetails) (((OAuth2Authentication) principal).getPrincipal())).getId();
        }
        throw new BackendRuntimeException(CommonConstant.EXCEPTION_MSG_LOGIN_USER_NOT_FOUND);
    }

    private String getToken(Principal principal) {
        if (principal instanceof OAuth2Authentication) {
            return ((OAuth2AuthenticationDetails) (((OAuth2Authentication) principal).getDetails())).getTokenValue();
        }
        throw new BackendRuntimeException(CommonConstant.EXCEPTION_MSG_LOGIN_USER_NOT_FOUND);
    }
}
