package cn.kdan.cloud.pdf.office.sso.controller;

import cn.kdan.cloud.pdf.office.api.account.feign.UserApi;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreateSubscriptionDTO;
import cn.kdan.cloud.pdf.office.api.payment.enums.PaymentMethodEnum;
import cn.kdan.cloud.pdf.office.api.payment.feign.OrderApi;
import cn.kdan.cloud.pdf.office.api.payment.feign.SubscriptionApi;
import cn.kdan.cloud.pdf.office.api.product.feign.ProductApi;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductVO;
import cn.kdan.cloud.pdf.office.common.constant.CommonConstant;
import cn.kdan.cloud.pdf.office.common.enums.ValidStatusEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.PDFOfficeUserStatusEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.PDFOfficeUserSubscriptionStatusEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.PayTypeEnum;
import cn.kdan.cloud.pdf.office.common.enums.payment.SubscriptionTypeEnum;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.pojo.CustomUserDetails;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import cn.kdan.cloud.pdf.office.common.utils.MyDateUtils;
import cn.kdan.cloud.pdf.office.common.vo.*;
import cn.kdan.cloud.pdf.office.sso.pojo.LogOutSystemDTO;
import cn.kdan.cloud.pdf.office.sso.service.AuthService;
import cn.kdan.cloud.pdf.office.sso.utils.RSAUtils;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.google.common.util.concurrent.ThreadFactoryBuilder;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.security.oauth2.provider.authentication.OAuth2AuthenticationDetails;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.security.Principal;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.stream.Collectors;

/**
 * 个人信息模块
 */
@RestController
@RequestMapping("/user")
public class UserController {

    private final RSAUtils rsaUtils = new RSAUtils();
    @Autowired
    private UserApi userApi;
    @Autowired
    private ProductApi productApi;
    @Autowired
    private SubscriptionApi subscriptionApi;
    @Autowired
    private OrderApi orderApi;
    @Autowired
    private AuthService authService;
    @Value("${avatarUrl}")
    private String avatarUrl;
    private static final ThreadPoolExecutor PAY_UPDATE_THREAD_POOL = new ThreadPoolExecutor( Runtime.getRuntime().availableProcessors(),
            Runtime.getRuntime().availableProcessors(),
            1L,
            TimeUnit.MINUTES,
            new LinkedBlockingQueue<>(300), new ThreadFactoryBuilder()
            .setNameFormat("PAY_NOTIFY_THREAD_POOL").build(), new ThreadPoolExecutor.AbortPolicy());

    public static String convertToLowerCase(String input) {
        if (input == null || input.isEmpty()) {
            return ""; // 如果输入为空，返回空字符串
        }

        // 分割字符串
        String[] platforms = input.split(",");

        // 将平台名称转换为小写
        StringBuilder lowerCasePlatforms = new StringBuilder();
        for (int i = 0; i < platforms.length; i++) {
            lowerCasePlatforms.append(platforms[i].toLowerCase()); // 转换为小写
            if (i < platforms.length - 1) {
                lowerCasePlatforms.append(","); // 添加逗号分隔符
            }
        }

        return lowerCasePlatforms.toString(); // 返回结果字符串
    }

    @GetMapping("/me")
    public UserInfoVO me(Principal principal) {
        if (principal instanceof OAuth2Authentication) {
            {
                String userId = ((CustomUserDetails) ((OAuth2Authentication) principal).getPrincipal()).getId();
                authService.refreshToken(principal);
                // 获取用户的基本信息
                UserInfoVO user = userApi.getInfoById(userId).getResult();
                user.getSubscriptionInfoList().forEach(item -> {
                    ProductVO vo = productApi.getProduct(item.getProductId()).getResult();
                    item.setProductCode(vo.getCode());
                });
                List<UserSubscriptionInfoVO> list = user.getSubscriptionInfoList().stream()
                        .sorted(Comparator.comparing(UserSubscriptionInfoVO::getPlatform, Comparator.reverseOrder()))
                        .collect(Collectors.toList());
                list.forEach(item -> {
                    //自动续订，显示续订日期
                    if (!ObjectUtils.isEmpty(item.getEndDate()) && PayTypeEnum.AUTO.value().equals(item.getPayType())) {
                        LocalDateTime payTime = subscriptionApi.getNextPayTime(item.getUserId(), item.getProductId()).getResult();
                        Date date = MyDateUtils.localDateTimeToDate(payTime);
                        item.setEndDate(date);
                    }
                    // 没有购买显示当前时间
                    if (ObjectUtils.isEmpty(item.getEndDate())) {
                        item.setEndDate(new Date());
                    }
                });
                user.setSubscriptionInfoList(list);
                user.setDigestPassword(null);
                return user;
            }
        }
        return null;
    }

    /**
     * 获取用户个人信息
     *
     * @return info
     */
    @GetMapping("/info")
    public ResultMap<MemberInfoVO> info(Principal principal,@RequestParam(required = false) String model) {
        if (principal instanceof OAuth2Authentication) {
            {
                String userId = ((CustomUserDetails) ((OAuth2Authentication) principal).getPrincipal()).getId();
                authService.refreshToken(principal);
                // 获取用户的基本信息
                UserInfoVO user = userApi.getMemberInfoById(userId).getResult();
                // 没有购买显示当前时间
                if (ObjectUtils.isEmpty(user.getFullName())) {
                    user.setFullName("");
                }
                MemberInfoVO result = new MemberInfoVO();
                BeanUtils.copyProperties(user, result);
                if (!StringUtils.isEmpty(user.getDigestPassword())) {
                    result.setIsHavePwd(ValidStatusEnum.VALID.value());
                }
                if (subscriptionApi.judgeAiDiscount(user.getId()).getResult()) {
                    result.setIsHaveAIDiscount(ValidStatusEnum.VALID.value());
                } else {
                    result.setIsHaveAIDiscount(ValidStatusEnum.INVALID.value());
                }
                if (subscriptionApi.judgeTrail(user.getId()).getResult()) {
                    result.setCanTrail(ValidStatusEnum.VALID.value());
                } else {
                    result.setCanTrail(ValidStatusEnum.INVALID.value());
                }
                UserSubscriptionInfoVO activeSubscription = authService.findActiveSubscription(user.getSubscriptionInfoList(), null);
                SimpleUserSubscriptionInfoVO vo = new SimpleUserSubscriptionInfoVO();
                 // 没有购买显示当前时间
                if (ObjectUtils.isEmpty(activeSubscription.getEndDate())) {
                    activeSubscription.setEndDate(new Date());
                }
                if (ObjectUtils.isEmpty(activeSubscription.getPoint())) {
                    activeSubscription.setPoint(0);
                }
                //默认不是ai
                activeSubscription.setIsAi(StringUtils.isEmpty(activeSubscription.getIsAi()) ? "0" : activeSubscription.getIsAi());
                BeanUtils.copyProperties(activeSubscription,vo);
                result.setActiveVIP(vo);
                vo.setPlatforms(convertToLowerCase(vo.getPlatforms()));
                if(activeSubscription.getStatus().equals(PDFOfficeUserSubscriptionStatusEnum.SUBSCRIPTION_IN_PROGRESS.value())
                        &&activeSubscription.getPlatform().equals(2)&&activeSubscription.getPayType().equals(PayTypeEnum.AUTO.value())) {
                    PAY_UPDATE_THREAD_POOL.submit(() -> {
                        orderApi.updateStatusPayment(userId, activeSubscription.getProductId());
                    });
                }
                result.setAiPoint(authService.getAiPoints(user.getSubscriptionInfoList()));
                SimpleUserSubscriptionInfoVO aiInfo = new SimpleUserSubscriptionInfoVO();
                UserSubscriptionInfoVO infoVO = authService.getActiveAi(user.getSubscriptionInfoList());
                BeanUtils.copyProperties(infoVO,aiInfo);
                result.setActiveAI(aiInfo);
                result.setAvatarUrl(avatarUrl);
                result.setIsHaveActiveSubscription(authService.judgeActiveSubscription(user.getSubscriptionInfoList()));
                return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, result);
            }
        }

        throw new BackendRuntimeException("net error");
    }

    /**
     * 获取用户当前的会员情况（用于用户注销时提醒用户）
     *
     * @return List
     */
    @GetMapping("/rightList")
    public ResultMap<List<SimpleUserSubscriptionInfoVO>> rightList(Principal principal) {
        if (principal instanceof OAuth2Authentication) {
            {
                String userId = ((CustomUserDetails) ((OAuth2Authentication) principal).getPrincipal()).getId();
                // 获取用户的基本信息
                UserInfoVO user = userApi.getMemberInfoById(userId).getResult();
                AtomicBoolean isUpgrade = new AtomicBoolean(false);
                List<SimpleUserSubscriptionInfoVO> resultList = user.getSubscriptionInfoList().stream().map(item -> {
                    if(item.getCode().contains("upgrade")&&item.getCode().contains("subscription")){
                        isUpgrade.set(true);
                    }
                    item.setPlatforms(convertToLowerCase(item.getPlatforms()));
                    // 没有购买显示当前时间
                    if (ObjectUtils.isEmpty(item.getEndDate())) {
                        item.setEndDate(new Date());
                    }
                    if (ObjectUtils.isEmpty(item.getPoint())) {
                        item.setPoint(0);
                    }
                    //默认不是ai
                    item.setIsAi(StringUtils.isEmpty(item.getIsAi()) ? "0" : item.getIsAi());
                    SimpleUserSubscriptionInfoVO vo = new SimpleUserSubscriptionInfoVO();
                    BeanUtils.copyProperties(item, vo);
                    return vo;
                }).collect(Collectors.toList());
                if (isUpgrade.get()) {
                    resultList.removeIf(info -> Objects.equals(info.getPlatforms(), "3")); // 通过 removeIf 来移除 level 为 0 的记录
                }
                if (resultList.size() > 1) {
                    resultList.removeIf(info -> StringUtils.isNotEmpty(info.getLevels())&&Objects.equals(info.getLevels(), "1")); // 通过 removeIf 来移除 level 为 0 的记录
                }
                return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, resultList);
            }
        }
        throw new BackendRuntimeException("net error");
    }

    /**
     * 登出(登出设备)
     *
     * @param deviceSign 设备号
     * @param appId      appId （PDF Reader Pro 为1）
     * @return Boolean
     */
    @PostMapping("/logout")
    public ResultMap<Boolean> logout(@RequestParam(required = false) String deviceSign, @RequestParam String appId, HttpServletRequest request, Principal principal) {
        String token = getToken(principal);
        String userId = getUserId(principal);
        authService.logout(token, userId, deviceSign, appId, true);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, Boolean.TRUE);
    }

    /**
     * 用户手动注销
     *
     * @param code  邮箱验证码
     * @param appId appId （PDF Reader Pro 为1）
     * @return Boolean
     */
    @PostMapping("/logOffForUser")
    public ResultMap<Map<String, String>> logOffForUser(@RequestParam String code, @RequestParam String appId, Principal principal) {
        String userId = getUserId(principal);
        userApi.logOffForUser(code, userId, appId).getResult();
        UserVO userVO = userApi.getById(userId).getResult();
        if (!appId.equals(CommonConstant.PDF_READER_PRO_APP_ID)) {
            authService.logOutAll(userVO);
        }
        LocalDateTime now = LocalDateTime.now();

        // 计算7天后的时间
        LocalDateTime sevenDaysLater = now.plusDays(7);
        // 定义日期格式
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        // 格式化当前时间和7天后的时间
        String currentTimeFormatted = now.format(formatter);
        String sevenDaysLaterFormatted = sevenDaysLater.format(formatter);
        Map<String, String> result = new HashMap<>();
        result.put("currentTime", currentTimeFormatted);
        result.put("logOffTime", sevenDaysLaterFormatted);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, result);
    }

    @PostMapping("/logOutAllSystem")
    public ResultMap<Boolean> logOutAllSystem(@RequestBody LogOutSystemDTO logOutSystemDTO) {
        UserVO userVO = userApi.getById(logOutSystemDTO.getUserId()).getResult();
        authService.logOutAll(userVO);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, Boolean.TRUE);
    }

    @PostMapping("/logOutSystem")
    public ResultMap<Boolean> logOutSystem(@RequestBody LogOutSystemDTO logOutSystemDTO) {
        authService.logOutDevice(logOutSystemDTO.getAppId(), logOutSystemDTO.getUserId(), logOutSystemDTO.getUniqueSn());
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, Boolean.TRUE);
    }

    /**
     * 撤销注销申请
     *
     * @return Boolean
     */
    @PostMapping("/revokeCancel")
    public ResultMap<Boolean> revokeCancel(Principal principal) {
        UserVO userVO = userApi.getById(getUserId(principal)).getResult();
        UserVO updateVO = new UserVO();
        updateVO.setId(userVO.getId());
        updateVO.setUpdatedAt(new Date());
        updateVO.setValidFlag(PDFOfficeUserStatusEnum.VALID.value().toString());
        userApi.updateUser(updateVO);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, Boolean.TRUE);
    }

    /**
     * 赠送7天试用
     *
     * @param appId appid 1
     * @return String
     */
    @PostMapping("/getTrail")
    public ResultMap<String> getTrail(@RequestParam String appId, Principal principal) {
        CreateSubscriptionDTO dto = new CreateSubscriptionDTO();
        dto.setStartDate(LocalDateTime.now());
        dto.setEndDate(LocalDateTime.now().plusDays(7));
        dto.setProductId("23");
        String userId = getUserId(principal);
        dto.setUserId(userId);
        dto.setAppId(appId);
        dto.setPayment(PaymentMethodEnum.NO_NEED_TO_PAY.getValue());
        UserInfoVO user = userApi.getMemberInfoById(userId).getResult();
        dto.setEmail(user.getEmail());
        dto.setSubscriptionType(SubscriptionTypeEnum.TRAIL.value());
        subscriptionApi.createSubscription(dto);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, CommonConstant.RESULT_SUCCESS);
    }

    @PostMapping("/getInviteNum")
    public ResultMap<Integer> getInviteNum( Principal principal) {
        String userId = getUserId(principal);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, userApi.getInviteNum(userId).getResult());
    }


    private String getUserId(Principal principal) {
        if (principal instanceof OAuth2Authentication) {
            return ((CustomUserDetails) (((OAuth2Authentication) principal).getPrincipal())).getId();
        }
        throw new BackendRuntimeException(CommonConstant.EXCEPTION_MSG_LOGIN_USER_NOT_FOUND);
    }

    private String getToken(Principal principal) {
        if (principal instanceof OAuth2Authentication) {
            return ((OAuth2AuthenticationDetails) (((OAuth2Authentication) principal).getDetails())).getTokenValue();
        }
        throw new BackendRuntimeException(CommonConstant.EXCEPTION_MSG_LOGIN_USER_NOT_FOUND);
    }
}
